/*
	File:		Core.r

	Contains:	nmap resources for OpenDoc

	Owned by:	Craig Carper

	Copyright:	 1993 - 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<16>	  9/7/95	CG		#1266854 FB:Bad Editor-alias warning.
		<15>	 8/30/95	CG		1257569 FB3:Added STR for OpenDoc prefs
									folder name.
		<14>	  8/8/95	jpa		Fixed layout/text of trashed-bad-alias
									alert. [1257556]
		<13>	 6/22/95	jpa		Added trashed-bad-alias alert [1257556]
		<12>	  6/8/95	RR		#1257260 Collapse base classes. #1214898
									StdTypes.r -> ODTypes.r
		<11>	 5/16/95	jpa		Added dialog kODScanEditorsDialogID. This
									is temporary. [1240150]
		<10>	 5/10/95	CG		1243604	The constant kNameMappings renamed
									to kODNameMappings.
		 <9>	 4/26/95	TJ		Removed version Resources, they are now in
									ODVersion.r
		 <8>	 3/29/95	NP		1233490: Use Mac system constant for
									international text stuff.
		 <7>	 3/21/95	T		1229970 Categories List needs to be updated
		 <6>	 2/16/95	T		#1220895 Core.r needs CategoryUserString
									nmap for Standard Categories.
		 <5>	  2/2/95	CG		#1207993, #1211081: Moved hard coded
									strings to resources.
		 <4>	 1/25/95	CC		1153802 Removed user string for 'styl'
									scrap type.
		 <3>	12/25/94	T		1191189 Eliminate duplicate Def.h, Def.xh
									files
		 <2>	 7/29/94	CG		Added vers 1 and 2 for use by prefs.
		 <1>	 7/11/94	T		first checked in
		 <0>	 7/11/94	SV		SOMverted
		 <2>	 3/18/94	CC		Include StdTypes.r to fix broken checkin.
									(1151640)
		 <1>	 3/18/94	CC		first checked in
		 <0>	 3/18/94	CC		Add user visible strings for Apple
		 							scrap types.  (1151640)

	To Do:
	In Progress:
		
*/


#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#ifndef __ODTYPES_R__
#include "ODTypes.r"
#endif

#ifndef _CONSTDEF_
#include "ConstDef.h"
#endif

#ifndef _PREFSDEF_
#include "PrefsDef.h"
#endif

resource 'STR ' (kODPrefsFileNameStrID, "Prefs file name") {
	"OpenDoc Preferences"
};

resource 'STR ' (kODEditorsFldrNameStrID, "Editors Folder name") {
	"Editors"
};

resource 'STR ' (kODLibsFldrNameStrID, "OD Libraries Folder name") {
	"OpenDoc Libraries"
};

resource 'STR ' (kODPrefsFolderNameStrID, "Prefs folder name") {
	"OpenDoc Prefs"
};


// Dialog to display while scanning Editors/Libraries folders.
// This is here as a semi-hack to tell when this happens and how long it takes.
// I'll leave it up to the HI team to decide whether it can be allowed to stay. --jpa
resource 'DLOG' (kODScanEditorsDialogID) {
	{40, 40, 92, 297},
	dBoxProc,
	visible,
	goAway,
	0x0,
	kODScanEditorsDialogID,
	"",
	alertPositionParentWindowScreen
};

resource 'DITL' (kODScanEditorsDialogID) {
	{	/* array DITLarray: 2 elements */
		/* [1] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		},
		/* [2] */
		{10, 65, 42, 259},
		StaticText {
			disabled,
			"Updating OpenDoc editors database..."
		}
	}
};


resource 'ALRT' (kODBadAliasInEditorsAlertID) {
	{44, 25, 173, 441},
	kODBadAliasInEditorsAlertID,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	alertPositionParentWindowScreen
};

resource 'DITL' (kODBadAliasInEditorsAlertID) {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{99, 342, 119, 400},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 62, 90, 400},
		StaticText {
			disabled,
			"The Editors folder can't contain aliases"
			" to items on other volumes. Aliases to e"
			"ditors on other volumes were found and w"
			"ere moved to the Trash. To use aliases, "
			"make sure the original items are on the "
			"same volume as the Editors folder."
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		},
		/* [4] */
		{96, 338, 122, 404},
		UserItem {
			disabled
		}
	}
};


// Standard Category 'nmap' and their user strings.

resource kODNameMappings (301) {
	kODCategoryUserString,
	{	/* array KeyList: Many elements */
		/* [1] */
		kODCategoryPlainText,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Plain Text"
		},
		
		/* [2] */
		kODCategoryStyledText,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Styled Text"
		},

		/* [3] */
		kODCategoryDrawing,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Drawing"
		},

		/* [4] */
		kODCategory3DGraphic,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"3D Graphic"
		},

		/* [5] */
		kODCategoryPainting,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Painting"
		},

		/* [6] */
		kODCategoryMovie,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Movie"
		},

		/* [7] */
		kODCategorySampledSound,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Sampled Sound"
		},

		/* [8] */
		kODCategoryChart,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Chart"
		},

		/* [9] */
		kODCategoryFormula,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Formula"
		},

		/* [10] */
		kODCategorySpreadsheet,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Spreadsheet"
		},

		/* [11] */
		kODCategoryTable,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Table"
		},

		/* [12] */
		kODCategoryDatabase,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Database"
		},

		/* [13] */
		kODCategoryQuery,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Query"
		},

		/* [14] */
		kODCategoryConnection,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Connection"
		},

		/* [15] */
		kODCategoryScript,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Script"
		},

		/* [16] */
		kODCategoryOutline,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Outline"
		},

		/* [17] */
		kODCategoryPageLayout,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Page Layout"
		},

		/* [18] */
		kODCategoryPresentation,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Presentation"
		},

		/* [19] */
		kODCategoryCalendar,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Calendar"
		},

		/* [20] */
		kODCategoryForm,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Form"
		},

		/* [21] */
		kODCategoryExecutable,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Executable"
		},

		/* [22] */
		kODCategoryCompressed,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Compressed"
		},

		/* [23] */
		kODCategoryControlPanel,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Control Panel"
		},

		/* [24] */
		kODCategoryControl,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Control"
		},

		/* [25] */
		kODCategoryPersonalInfo,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Personal Info"
		},

		/* [26] */
		kODCategorySpace,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Space"
		},

		/* [27] */
		kODCategoryProject,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Project"
		},

		/* [28] */
		kODCategorySignature,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Signature"
		},

		/* [29] */
		kODCategoryKey,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Key"
		},

		/* [30] */
		kODCategoryUtility,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Utility"
		},

		/* [31] */
		kODCategoryMailingLabel,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Mailing Label"
		},

		/* [32] */
		kODCategoryLocator,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Locator"
		},

		/* [33] */
		kODCategoryPrinter,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Printer"
		},

		/* [34] */
		kODCategoryStructuredSound,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Structured Sound"
		},

		/* [n] */
		kODCategoryTime,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Time"
		}
	}
};
