/*
	File:		ShPrpAcc.h

	Contains:	Interfaces to property accessor classes used with Object
				Support Library

	Owned by:	Eric House

	Copyright:	 1993 - 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <4>	 8/22/95	eeh		1230007: refbal-inspired cleanup
		 <3>	 2/20/95	T		#1210979 BB: 5-$ Bugs need to be evaluated
									and removed from Shell
		 <2>	 8/19/94	NP		1181622: Ownership fix.
		 <1>	 6/30/94	T		first checked in
		 <0>	 6/29/94	SV		SOMverted
		 <7>	 3/24/94	eeh		bug# 1151750: scripting support for
									operations on draft
		 <6>	  2/8/94	JA		Minor tweaks for CodeWarrior.
		 <5>	 1/11/94	eeh		removed "frame" from objspec vocab
		 <4>	12/17/93	eeh		more tweaks....
		 <3>	 12/9/93	eeh		new classes: ShellSelfPropAccessor and
									ShellFramePropAccessor
		 <2>	11/15/93	eeh		added GetPropertyData and SetPropertyData
		 <1>	10/11/93	NP		first checked in

	To Do:
*/

#ifndef _SHPRPACC_
#define _SHPRPACC_

#ifndef _PLFMDEF_
#include "PlfmDef.h"
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

/*
	These classes aid with the getting and setting of properties for
	the callback routines for the Object Support Library. They encapulate the
	information necessary to represent a property and to get and set the
	property's value.
*/

//==============================================================================
// Classes defined in this interface
//==============================================================================

// class ShellWinPropAccessor;

//==============================================================================
// Classes used by this interface
//==============================================================================

class PropAccessor;
class RealShell;
//class ODWindow;

//==============================================================================
// ShellWinPropAccessor
//==============================================================================

#ifdef TO_BE_DELETED
class ShellPropAccessor
{
	public:
		ShellPropAccessor( DescType property, RealShell* shell ) ;

		virtual void GetPropertyData( AEDesc* result ) = 0;
		virtual void SetPropertyData( AEDesc* data ) = 0;

		DescType	GetProperty();
		RealShell*	GetShell() ;

protected :
	DescType	fProperty;

  private:

	RealShell*	fShell;
} ;
#endif /* TO_BE_DELETED */

#ifdef TO_BE_DELETED
class ShellSelfPropAccessor : public ShellPropAccessor
{
	public:
		ShellSelfPropAccessor( DescType property, RealShell* shell ) ;

		void GetPropertyData( AEDesc* result ) ;
		void SetPropertyData( AEDesc* data ) ;
} ;

class ShellPartPropAccessor : public ShellPropAccessor
{
	public:
		ShellPartPropAccessor( DescType property, RealShell* shell,
				ODFrame* frame) ;

		void GetPropertyData( AEDesc* result ) ;
		void SetPropertyData( AEDesc* data ) ;

  private:
	ODFrame* fFrame ;
} ;
#endif /* TO_BE_DELETED */

#ifdef TO_BE_DELETED
class ShellFramePropAccessor : public ShellPropAccessor
{
	public:
		ShellFramePropAccessor( DescType property, RealShell* shell,
				ODFrame* frame) ;

		void GetPropertyData( AEDesc* result ) ;
		void SetPropertyData( AEDesc* data ) ;

  private:
	ODFrame* fFrame ;
} ;
#endif // TO_BE_DELETED

#ifdef TO_BE_DELETED
class ShellDraftPropAccessor : public ShellPropAccessor
{
	public:
		ShellDraftPropAccessor( DescType property, RealShell* shell,
				ODDraft* draft ) ;

		void GetPropertyData( AEDesc* result ) ;
		void SetPropertyData( AEDesc* data ) ;

  private:
	ODDraft* fDraft ;
} ;

class ShellWinPropAccessor : public ShellPropAccessor
{
  public:

	ShellWinPropAccessor(DescType property, RealShell* shell, ODWindow* window);

	void GetName(StringPtr name);
		// Caller must allocate storage

	void GetPropertyData( AEDesc* result ) ;
	void SetPropertyData( AEDesc* data ) ;

  private:
	ODWindow*	fWindow;
};
#endif /* TO_BE_DELETED */

#endif // _SHPRPACC_