/*
	File:		Shell.r

	Contains:	Resources for the document shell

	Owned by:	Nick Pilch

	Copyright:	 1993 - 1996 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <5>	 3/29/96	DM		1334273: added new alert 493 which
									announces OpenDoc must quit immediately due
									to very low memory.
		 <4>	.03.1996	NP		1307182: Removed STR# item.
		 <3>	 3/14/96	NP		1330149: Remove objectionable MENU entries.
		 <2>	 1/15/96	JA		1269485: Resized About dlog & added 'ictb'
									to change font.
		<43>	10/31/95	eeh		1297192: make room for strings in dialogs
		<42>	10/31/95	TJ		Made the Splash Screen Non-Modal.#1297413
		<41>	10/26/95	T		1296574 GM:Cannot set default memory
									partition size.
		<40>	10/25/95	T		1281409 GM: OpenDoc needs an AboutBox.
		<39>	10/24/95	jpa		1293441: Added more lo-mem alerts.
		<38>	10/18/95	CC		GM 1293627: Use
									alertPositionParentWindowScreen for all
									alerts.
		<37>	10/16/95	CC		GM 1291511: Layout changes to Save As
									dialog to hide Kind popup.
		<36>	 10/4/95	jpa		CmmnUtls.r --> UtilCmmn.r [1285767]
		<35>	 9/29/95	eeh		1287262: include CmmnUtls.r, remove quotes
		<34>	 9/27/95	eeh		1284893: fix capitalization
		<33>	 9/20/95	T		1285201 GM: Use
									alertPositionParentWindowScreen for dialogs
		<32>	 9/12/95	JBS		1282526 Status message needed during "Save
									Doc to another volume"
		<31>	 9/12/95	eeh		1253309: add kSHLPluginErrorStrID string
		<30>	  9/8/95	T		1252120 FB1: New documents saved w/ StdFile
									& trashed if closed w/o saving
		<29>	  9/8/95	eeh		1268998: add prefs file name 'STR '
									resource
		<28>	  9/7/95	eeh		1278987: 'STR ' resources for UniquifyName
		<27>	 8/22/95	eeh		1276281: template strings
		<26>	 8/15/95	T		1258489 Remove TM from OpenDoc folder
		<25>	  8/8/95	jpa		Added kSHLDragToTrashAlert (commented out)
									[1259401]
		<24>	 6/20/95	T		1217754 BB: Delete Document command
		<23>	  6/8/95	RR		#1257260 Collapse base classes. #1214898
									StdTypes.r -> ODTypes.r
		<22>	  6/1/95	jpa		Added ^2 to error DITL so we can show
									exception message str. [1242839]
		<21>	 5/19/95	T		1155852 BB: String constants need to be
									moved to resource for Localization
		<20>	  5/7/95	T		1155852 BB: String constants need to be
									moved to resource for Localization (mostly
									done)
		<19>	  5/4/95	jpa		Changed text of low-mem alert and added app
									low-mem alert [1235657]
		<18>	  5/1/95	CL		Change Revert and Save Changes ALRTs to
									DLOGs, add STR# to support cmd-keys for
									buttons [1228662,1228678,1229912].
		<17>	 4/26/95	TJ		Removed version Resources, they are now in
									ODVersion.r
		<16>	 4/13/95	CG		#1221869: Added STR resources for Shell
									Plug-Ins folder name.
		<15>	 4/10/95	T		#1236210	BB: "Show Parts Bin" menu item
									must be removed from the Edit menu
		<14>	 2/20/95	T		#1210979 BB: 5-$ Bugs need to be evaluated
									and removed from Shell
									#1221861 BB: Incomplete AOCE support needs
									to be removed from OpenDoc
		<13>	 2/16/95	CG		#1191912: Changed Print to Print
		<12>	  1/6/95	jpa		Read DocStub.rsrc, not DocStub.r.
		<11>	12/25/94	T		1191189 Eliminate duplicate Def.h, Def.xh
									files
									1187709 More frequent, robust and earlier
									error detection and reporting in the Shell
									Moved Error message resources to ErrMsgs.r.
		<10>	11/17/94	CG		(for TC) 1194570, 1193996, 1194812, 1191912
									- user experience fixes.
		 <9>	 10/7/94	T		#1187826 Document Info functionality
		 <8>	 9/19/94	T		#1150345 Shell.r does not need to refer to
									Test Parts
		 <7>	 9/16/94	eeh		#1186754: eliminate non-AOCE document menu.
		 <6>	 8/25/94	T		#1183223 Enabled launching of documents in
									folders other than that of the OpenDoc
									libraries.  Added document stub resources.
		 <5>	 8/19/94	T		#1159233 Remove references to j3di
		 <4>	  8/8/94	jpa		Added "Heap Info" dialog [1179567]
		 <3>	 7/21/94	T		#define oldTemp // to get 'dctb' resources
									to rez
		 <2>	 7/21/94	eeh		fix to comments
		 <1>	  7/6/94	T		first checked in
		 <0>	  7/6/94	SV		SOMverted
		<39>	 4/11/94	T		#1155854 Use CustomPutFile for Save A Copy.
		<38>	  4/7/94	T		fixed #1155956
		<37>	 3/23/94	CG		#1151393: Use OpenDoc-wide consts for
									'vers' 1 and 2 resources.
		<36>	  3/9/94	eeh		bug #1149630: reversed "Forward" and
									"Reply" in mailer menu.
		<35>	 2/15/94	T		fix #1144029 and some cleanup
		<34>	 1/24/94	NP		Changed DITL for About Box to enable the
									PICT item.
		<33>	 1/19/94	eeh		added Open Document menu item
		<32>	 1/18/94	T		removed scsz resource for AppleScript, the
									Shell library does not need an scsz
									resource, only the OpenDoc extension and
									documents do.
		<31>	 1/18/94	T		use kOpenDocShellSignature & remove BNDL &
									FREF resources (why were they there?)
		<30>	 1/13/94	eeh		fixed name of document menu
		<29>	 1/11/94	eeh		make mailer menu heierarchical
		<28>	12/17/93	eeh		changed dontReadExtensionTerms to
									readExtensionTerms in 'scsz'
		<27>	 12/9/93	eeh		changed rezid of 'scsz' to 0
		<26>	 12/3/93	T		Rename ODError.h to ErrorDef.h
		<25>	11/30/93	NP		Added alwaysSendSubject to scsz resource.
		<24>	11/24/93	T		remove comments about SIZE -1 resource
		<23>	11/15/93	eeh		removed 'aete' resource; added menus for
									Reply and Forward (mailer)
		<22>	11/12/93	CG		Added scsz resource for AppleScript.
		<21>	10/27/93	T		update type 'errs' for the change of
									ODError from ODULong to ODSLong
		<20>	10/20/93	eeh		Readded expand- and contract-mailer menus
									(for use during development only).
		<19>	10/18/93	T		moved SIZE -1 resource to Document.r
		<18>	10/12/93	eeh		changes to AOCE menu
		<17>	 10/8/93	T		bifurcated Document menu into normal and
									AOCE versions
		<16>	 10/7/93	VL		Added more error codes.
		<15>	 10/7/93	eeh		various changes related to adding Mailer
									support (more to come)
		<14>	 10/1/93	RR		Fix file corruption
		<13>	 10/1/93	T		fixed definition of 'errs' resource.
									UNSIGNED LONGINT for fields
		<12>	 9/30/93	T		changed 'errs' resource to use LONGINTs for
									error numbers and ranges
		<11>	 9/22/93	NP		Changed aete resource.
		<10>	 9/17/93	T		changed 'File' menu to 'Document'
		 <9>	 9/15/93	T		added Error Handling resources, 'errs' etc.
		 <8>	 9/10/93	RR		Remove File and Edit from MBAR resource
		 <7>	 8/27/93	T		added About Box from Prototype
		 <6>	 8/19/93	T		updated Menus for HI Changes
		 <5>	  8/4/93	T		added AETE resource
		 <4>	 7/21/93	T		updated standard DocShell menus
		 <3>	 5/24/93	T		add Save Changes alert (stolen from
									TeachText) and Revert alert
		 <2>	 4/30/93	T		isStationeryAware
		 <1>	 4/27/93	RCR		First Checked In.

	To Do:
	In Progress:
		
*/

#ifndef _SHELLDEF_
#include "ShellDef.h"
#endif

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include "StdDefs.r"
#endif



#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#define oldTemp			// to get 'dctb' resources to rez
#ifndef __TYPES_R__
#include "Types.r"
#endif
#undef oldTemp

#ifndef __SYSTYPES_R__
#include "SysTypes.r"
#endif

#ifndef _CONSTDEF_
#include "ConstDef.h"
#endif

#ifndef _UTILDEFS_
#include "UtilDefs.h"
#endif

include "DocStub.rsrc";					// Include document-stub resources


#define	rMenuBar	128				/* application's menu bar */
#define	rAboutAlert	128				/* about alert */
#define	rDocWindow	128				/* application's window */

/* The following constants are used to identify menus and their items. The menu IDs
   have an "m" prefix and the item numbers within each menu have an "i" prefix. */
#define	mApple					255		/* Apple menu */
#define	iAbout					1

#define	mFile					256		/* File menu */

#define	mEdit					257		/* Edit menu */


resource 'MBAR' (255, preload) {
	{ mApple  };		/* File and edit are added later */
};


resource 'MENU' (mApple, preload) {
	mApple, textMenuProc,
	0b1111111111111111111111111111101,	/* disable dashed line, enable About and DAs */
	enabled, apple,
	{
		"About OpenDoc",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain
	}
};

resource 'MENU' (mFile, preload) {
	mFile, textMenuProc,
	0b0000000000000000001011111010101,	
	enabled, "Document",
	{
		"New",
			noicon, "N", nomark, plain;
		"Open Selection",
			noicon, nokey, nomark, plain;
		"Open Document",
			noicon, "O", nomark, plain;
		"Insert",
			noicon, nokey, nomark, plain;
		"Close",
			noicon, "W", nomark, plain;
		"Delete",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Save",
			noicon, "S", nomark, plain;
		"Save a Copy",
			noicon, nokey, nomark, plain;
		"Revert to Saved",
			noicon, nokey, nomark, plain;
		"Drafts",
			noicon, nokey, nomark, plain;
		"Document Info",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Page Setup",
			noicon, nokey, nomark, plain;
		"Print",
			noicon, "P", nomark, plain
	}
};

resource 'MENU' (mEdit, preload) {
	mEdit, textMenuProc,
	0b0000000000000000000000000000000,	/* disable everything, program does the enabling */
	enabled, "Edit",
	 {
		"Undo",
			noicon, "Z", nomark, plain;
		"Redo",
			noicon, "R", nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Cut",
			noicon, "X", nomark, plain;
		"Copy",
			noicon, "C", nomark, plain;
		"Paste",
			noicon, "V", nomark, plain;
		"Paste As",
			noicon, nokey, nomark, plain;
		"Clear",
			noicon, nokey, nomark, plain;
		"Select All",
			noicon, "A", nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Part Info",
			noicon, "L", nomark, plain;
		"Preferences",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"View in Window",
			noicon, nokey, nomark, plain
	}
};

/* saving to a different volume status dialog */
	/* $$$$$ fix coordinates below */

resource 'DLOG' (kSHLSaveDiffVolDialogID) {
	{40, 40, 92, 297},
	dBoxProc,
	visible,
	goAway,
	0x0,
	kSHLSaveDiffVolDialogID,
	"",
	alertPositionParentWindowScreen
};

resource 'DITL' (kSHLSaveDiffVolDialogID) {
	{	/* array DITLarray: 2 elements */
		/* [1] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		},
		/* [2] */
		{10, 65, 42, 259},
		StaticText {
			disabled,
			"Saving Document..."	/* $$$$$ should have document name */
		}
	}
};


/* "Do you want to save changes?" dialog */
resource 'DLOG' (kSHLsvChanges) {
	{94, 80, 211, 434},	
	dBoxProc, invisible, noGoAway, 0,
	kSHLsvChanges,
	"",
	alertPositionParentWindowScreen
};

/* Item list for "Do you want to save changes?" dialog 			 */
/* Note: Strings in the STR# following this DITL correspond to	 */
/* Each DITL item.  Strings represent the command key code       */
/* for buttons in this DITL.									 */
resource 'DITL' (kSHLsvChanges) {
	{	/* array DITLarray: 5 elements */
		/* [1] */
		{87, 284, 107, 344},
		Button {
			enabled,
			"Save"
		},
		/* [2] */
		{87, 211, 107, 271},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{87, 70, 107, 155},
		Button {
			enabled,
			"Dont Save"
		},
		/* [4] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		},
		/* [5] */
		{10, 72, 76, 344},
		StaticText {
			disabled,
			"Save changes to the document ^0 before closing?"
		}
	}
};


/* Command-key codes for corresponding dialog items in Save Changes */
resource 'STR#' ( kSHLsvChangesCmdKeyStrs, purgeable ) {
	{	"s",	/* Save button 		 */
		".",	/* Cancel button 	 */
		"d"		/* Don't Save button */	}
};


/* Document-dragged-to-Trash alert: */

/* Currently no alert is used. If it will be, uncomment this resource. */
#if 0
	resource 'ALRT' (kSHLDragToTrashAlert, "Trash Alert", purgeable) {
		{40, 40, 210, 430},
		kSHLDragToTrashAlert,
		beepStages, 
		alertPositionParentWindowScreen
	};

	resource 'DITL' (kSHLDragToTrashAlert, "Trash Alert") {
		{
			{140, 310, 160, 370},	Button { enabled, "Save" },
			{140, 70, 160, 150},	Button { enabled, "Don't Save" },
			{140, 160, 160, 300},	Button { enabled, "Remove From Trash" },
			{10, 20, 42, 52},		Icon   { disabled, 2 },
			{10, 70, 130, 370},		StaticText { disabled,
				"The document ^0 has been dragged to the trash. Changes have been made "
				"since the last time this document was saved. \n\nSave changes to this "
				"document before closing it?"
			}
		}
	};
#endif


/* "Revert to the last saved version?" dialog */
resource 'DLOG' (kSHLsvRevert) {
	{94, 80, 194, 380},
	dBoxProc, invisible, noGoAway, 0,
	kSHLsvRevert,
	"",
	alertPositionParentWindowScreen
};

/* Item list for Revert dialog 									 */
/* Note: Strings in the STR# following this DITL correspond to	 */
/* Each DITL item.  Strings represent the command key code       */
/* for buttons in this DITL.									 */
resource 'DITL' (kSHLsvRevert) {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{73, 230, 93, 290},
		Button {
			enabled,
			"Revert"
		},
		/* [2] */
		{73, 157, 93, 217},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		},
		/* [4] */
		{10, 71, 61, 300},
		StaticText {
			disabled,
			"Revert to the last saved version of the document ^1?"
		}
	}
};


/* Command-key codes for corresponding dialog items in Revert Dialog */
resource 'STR#' ( kSHLsvRevertCmdKeyStrs, purgeable ) {
	{	"r",	/* Revert button */
		"."		/* Cancel button */	}
};

/* "Delete this document?" dialog */
resource 'DLOG' (kSHLDeleteDLOG) {
	{94, 80, 194, 380},
	dBoxProc, invisible, noGoAway, 0,
	kSHLDeleteDLOG,
	"",
	alertPositionParentWindowScreen
};

/* Item list for Delete dialog 									 */
/* Note: Strings in the STR# following this DITL correspond to	 */
/* Each DITL item.  Strings represent the command key code       */
/* for buttons in this DITL.									 */
resource 'DITL' (kSHLDeleteDLOG) {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{73, 230, 93, 290},
		Button {
			enabled,
			"Delete"
		},
		/* [2] */
		{73, 157, 93, 217},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		},
		/* [4] */
		{10, 71, 61, 300},
		StaticText {
			disabled,
			"Delete the document ^1?"
		}
	}
};


/* Command-key codes for corresponding dialog items in Revert Dialog */
resource 'STR#' ( kSHLDeleteDLOGCmdKeyStrs, purgeable ) {
	{	"d",	/* Delete button */
		"."		/* Cancel button */	}
};


/* Save a copy dialog */
resource 'DLOG' (kSHLSaveCopyDlgID, "Save A Copy", purgeable) {
	{0, 0, 178+11+19+7, 344},
	dBoxProc, invisible, noGoAway, 0,
	kSHLSaveCopyDlgID, 
	"",
	alertPositionParentWindowScreen
};

resource 'DITL'(kSHLSaveCopyDlgID, "Save A Copy") {	
	{
		/*   The first 12 items are from the default Save dialog
		 *   as described on p.age 3-18 and 3-19 of Inside Mac:Files
		 */ 
		/* sfItemOpenButton */
		{154+16+12, 252, 174+16+12, 332}, Button { enabled, "Save" },
		/* sfItemCancelButton */
		{123+12+12, 252, 143+12+12, 332}, Button { enabled, "Cancel" },
		/* sfItemBalloonHelp */
		{0, 0, 0, 0}, UserItem { disabled }, /* HelpItem { disabled, HMScanhdlg {-6043}},*/
		/* sfItemVolumeUser */
		{8, 235, 24, 337}, UserItem { enabled },
		/* sfItemEjectButton */
		{32, 252, 52, 332}, Button { enabled, "Eject" },
		/* sfItemDesktopButton */
		{60, 252, 80, 332}, Button { enabled, "Desktop" },
		/* sfItemFileListUser */
		{29, 12, 127, 230}, UserItem { enabled },
		/* sfItemPopUpMenuUser */
		{6, 12, 25, 230}, UserItem { enabled },
		/* sfItemDividerLinePict */
		{115+6+6, 250, 116+6+6, 334}, Picture { disabled, 11 },
		/* sfItemFileNameTextEdit */
		{150, 15, 150+16, 227}, EditText { enabled, "" },
		/* sfItemPromptStaticText */
		{129, 15, 129+16, 227}, StaticText { disabled, "Save as:" },
		/* sfItemNewFolderUser */
		{88, 252, 108, 332}, UserItem { disabled },
		/* 
		 *   New items specific to OpenDoc follow
		 */
		/* kSHLSaCDocumentRadioBtn */
		{166+18, 15, 166+18+16, 120}, RadioButton {enabled, "    Document"},
		/* kSHLSaCStationeryRadioBtn */
		{166+18, 125, 166+18+16, 227}, RadioButton {enabled, "    Stationery"},
		/* kSHLSaCKindPopupItem */
		{166+11+16+8+1000, 15-3, 166+11+16+8+19+1000, 227+3}, Control {disabled, kSHLSaCKindPopupItem},
		/* kSHLSaCDocumentIconItem */
		{166+17, 15+17, 166+17+16, 15+17+16}, UserItem {disabled},
		/* kSHLSaCStationeryIconItem */
		{166+17, 125+17, 166+17+16, 125+17+16}, UserItem {disabled}
	}
};


/* Dialog color table -- allows CustomPutFile to use color */
resource 'dctb' (kSHLSaveCopyDlgID, "Save a Copy") {
	0,
	0,
	{
		wContentColor,0xFFFF,0xFFFF,0xFFFF
	}
};


resource 'CNTL' (kSHLSaCKindPopupItem) {
		{173+11, 15-3, 173+11+19, 227+3},
		popupTitleRightJust,
		visible,
		40,					/* width of title in pixels     */
		kSHLSaCKindPopupMenu,				/* resource id of MENU resource */
		popupMenuCDEFproc+popupUseWFont+popupFixedWidth,
		0,					/* Refcon */
		"Kind:"
};

resource 'MENU' (kSHLSaCKindPopupMenu, "Kind") {
	kSHLSaCKindPopupMenu,
	textMenuProc,
	allEnabled,
	enabled,
	"Kind",
	{	/* array: 1 elements */
		"Translate to", noIcon, noKey, noMark, italic
	}
};


resource 'ics8' (kSHLSaCDocumentIconID) {
	$"00FF FFFF FFFF FFFF FFFF FF00 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FFFF 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FF2B FF00 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FFFF FFFF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 FFF5 FFFF F5FF FFF5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 FFFF FFFF F5FF FFF5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 FFFF F5FF FFF5 FFF5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'ics8' (kSHLSaCStationeryIconID) {
	$"00FF FFFF FFFF FFFF FFFF FFFF 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF FFFF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF F9FF 0000"
	$"00FF F5F5 FFF5 FFFF F5FF F5FF F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF F9FF 0000"
	$"00FF F5F5 FFFF FFFF F5FF F5FF F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF F9FF 0000"
	$"00FF F5F5 FFFF F5FF FFF5 F5FF F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5FF F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 FFFF FFFF F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 FFF6 FFF9 F9FF 0000"
	$"00FF F5F5 F5F5 F5F5 FFFF F9F9 F6FF 0000"
	$"00FF FFFF FFFF FFFF FFF9 F9F6 F6FF 0000"
	$"0000 00FF F9F9 F9F9 F9F9 F6F6 F6FF 0000"
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF"
};

resource 'ics4' (kSHLSaCDocumentIconID) {
	$"0FFF FFFF FFF0 0000 0F00 0000 00FF 0000"
	$"0F00 0000 00FC F000 0F00 0000 00FF FF00"
	$"0F00 0000 0000 0F00 0F00 0000 0000 0F00"
	$"0F00 F0FF 0FF0 0F00 0F00 0000 0000 0F00"
	$"0F00 FFFF 0FF0 0F00 0F00 0000 0000 0F00"
	$"0F00 FF0F F0F0 0F00 0F00 0000 0000 0F00"
	$"0F00 0000 0000 0F00 0F00 0000 0000 0F00"
	$"0F00 0000 0000 0F00 0FFF FFFF FFFF FF"
};

resource 'ics4' (kSHLSaCStationeryIconID) {
	$"0FFF FFFF FFFF 0000 0F00 0000 000F 0000"
	$"0F00 0000 000F FF00 0F00 0000 000F DF00"
	$"0F00 F0FF 0F0F DF00 0F00 0000 000F DF00"
	$"0F00 FFFF 0F0F DF00 0F00 0000 000F DF00"
	$"0F00 FF0F F00F DF00 0F00 0000 000F DF00"
	$"0F00 0000 FFFF DF00 0F00 0000 FCFD DF00"
	$"0F00 0000 FFDD CF00 0FFF FFFF FDDC CF00"
	$"000F DDDD DDCC CF00 000F FFFF FFFF FF"
};

resource 'ics#' (kSHLSaCDocumentIconID) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FE0 4030 4028 403C 4004 4004 4B64 4004"
		$"4F64 4004 4DA4 4004 4004 4004 4004 7FFC",
		/* [2] */
		$"7FE0 7FF0 7FF8 7FFC 7FFC 7FFC 7FFC 7FFC"
		$"7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC"
	}
};

resource 'ics#' (kSHLSaCStationeryIconID) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FF0 4010 401C 4014 4B54 4014 4F54 4014"
		$"4D94 4014 40F4 40A4 40C4 7F84 1004 1FFC",
		/* [2] */
		$"7FF0 7FF0 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC"
		$"7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 1FFC 1FFC"
	}
};



//-------------------------------------------------------------------------------------
// Folder names
//-------------------------------------------------------------------------------------


resource 'STR ' (kODEditorsFldrStrID, "Editors Folder name") {
	"Editors"
};

resource 'STR ' (kODStationeryFldrStrID, "Stationery Folder name") {
	"Stationery"
};

resource 'STR ' (kODOpenDocPartsFolderStrID, "OpenDoc Parts Folder name") {
	"OpenDoc"
};

resource 'STR ' (kODShellPlugInFolderStrID, "OD Shell Plug Ins Folder name") {
	"OpenDoc Shell Plug-Ins"
};

// Note: the following two resources were copied from Core.r.
// A copy of these resources also exists in OpenDocX.r

resource 'STR ' (kSHLPrefsFileNameResID, "ODPrefs file name") {
	"OpenDoc Preferences"
};

resource 'STR ' (kSHLPrefsFolderNameResID, "ODPrefs folder name") {
	"OpenDoc Prefs"
};

//-------------------------------------------------------------------------------------
// New filename construction template
//-------------------------------------------------------------------------------------

// Shell new filename construction template
// ^0 is the text seed (like "untitled"), ^1 is the date (like "5/18/95")
// Localizers can change string delimiter & order, and can add suffix or prefix
// Note that string will be truncated from end when a number is added to achieve
// filename uniqueness.

resource 'STR ' (kSHLNewFilenameTemplateStrID, "") {
	"^0 ^1"
};

#include "UtilCmmn.r"

//-------------------------------------------------------------------------------------
// Save, Close and Delete menuitem text construction templates
//-------------------------------------------------------------------------------------

resource 'STR ' (kSHLSaveItemTemplateStrID, "") {
	"Save ^0"
};

resource 'STR ' (kSHLCloseItemTemplateStrID, "") {
	"Close ^0"
};

resource 'STR ' (kSHLDeleteItemTemplateStrID, "") {
	"Delete ^0"
};

resource 'STR ' (kSHLPlugInErrorStrID, "") {
	"The OpenDoc Shell Plug-Ins ^0 failed to install properly due to the "
	"following error: ^1 Try removing it from your Shell Plug-Ins folder."
};


//-------------------------------------------------------------------------------------
// Error handling resources
//-------------------------------------------------------------------------------------

/*
	OpenDoc Error Resources
	based on MacApp ErrorMgr resources
*/

/* NOTE:

	By Toolbox convention, the OK button is item 1 and the Cancel button
		is item 2.
*/

resource 'ALRT' (kSHLphGenError,purgeable) {
	{80, 80, 258, 462},
	kSHLphGenError,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1;
		/* [2] */
		OK, visible, sound1;
		/* [3] */
		OK, visible, sound1;
		/* [4] */
		OK, visible, sound1
	}
#if SystemSevenOrLater
	,
	alertPositionParentWindowScreen
#endif
};

resource 'DITL' (kSHLphGenError,purgeable) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{140, 258, 158, 332},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		/*	^0 = userfriendly reason for error and how to recover; 
				e.g., 'The disk is locked.  Eject the disk and move the tab.'
			or
				technically detailed reason for error
				e.g., 'Volume error.'
		*/
		{10, 70, 135, 332},
		StaticText {
			disabled,
			"^0\n^2"		/* ^2 is the exception message str in the technical err alert. */
		},
		/* [3] */
		/*	^1 = exception number for error; 
				e.g., 'Exception: 5'
		*/
		{140, 70, 158, 250},
		StaticText {
			disabled,
			"^1"
		},
		/* [4] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			0
		}
	}
};

/* Used when free memory is very low */

resource 'ALRT' (kSHLphSpaceIsLow) {
	{49, 40, 189 /*was 174*/, 370},
	kSHLphSpaceIsLow,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1;
		/* [2] */
		OK, visible, sound1;
		/* [3] */
		OK, visible, sound1;
		/* [4] */
		OK, visible, sound1
	},
	alertPositionParentWindowScreen
};

resource 'DITL' (kSHLphSpaceIsLow) {
	{	/* array DITLarray: 5 elements */
		/* [1] */
		{103 /*was 88*/, 250, 123 /*was 108*/, 310},
		Button {
			enabled,
			"Save"
		},
		/* [2] */
		{103 /*was 88*/, 177, 123 /*was 108*/, 237},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{103 /*was 88*/, 36, 123 /*was 108*/, 121},
		Button {
			enabled,
			"Dont Save"
		},
		/* [4] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		},
		/* [5] */
		{10, 70, 92 /*was 77*/, 310},
		StaticText {
			disabled,
			"OpenDoc is almost out of memory. We reco"
			"mmend that you close this document immediatel"
			"y. Closing other windows or applications"
			" can also free up memory."
		}
	}
};


resource 'ALRT' (kSHLphAppSpaceIsLow) {
	{55, 49, 208 /*was 193*/, 379},
	kSHLphAppSpaceIsLow,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1;
		/* [2] */
		OK, visible, sound1;
		/* [3] */
		OK, visible, sound1;
		/* [4] */
		OK, visible, sound1
	},
	alertPositionParentWindowScreen
};

resource 'DITL' (kSHLphAppSpaceIsLow) {
	{	/* array DITLarray: 5 elements */
		/* [1] */
		{118 /*was 103*/, 250, 138 /*was 123*/, 310},
		Button {
			enabled,
			"Save"
		},
		/* [2] */
		{118 /*was 103*/, 177, 138 /*was 123*/, 237},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{118 /*was 103*/, 36, 138 /*was 123*/, 121},
		Button {
			enabled,
			"Dont Save"
		},
		/* [4] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		},
		/* [5] */
		{10, 70, 108 /*was 93*/, 310},
		StaticText {
			disabled,
			"This document is almost out of memory."
			" We recommend that you close it immediately or"
			" close other windows. Use the Document Info command"
			" to increase the amount of memory reserved for it."
		}
	}
};


resource 'ALRT' (kSHLphSpaceIsLowNoSave) {
	{49, 40, 189 /*was 174*/, 370},
	kSHLphSpaceIsLowNoSave,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	alertPositionParentWindowScreen
};

resource 'DITL' (kSHLphSpaceIsLowNoSave) {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{103 /*was 88*/, 250, 123 /*was 108*/, 310},
		Button {
			enabled,
			"Close"
		},
		/* [2] */
		{103 /*was 88*/, 177, 123 /*was 108*/, 237},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		},
		/* [4] */
		{10, 70, 92 /*was 77*/, 310},
		StaticText {
			disabled,
			"OpenDoc is almost out of memory. We reco"
			"mmend that you close this document immediatel"
			"y. Closing other windows or applications"
			" can also free up memory."
		}
	}
};

resource 'ALRT' (kSHLphAppSpaceIsLowNoSave) {
	{55, 49, 208 /*was 193*/, 379},
	kSHLphAppSpaceIsLowNoSave,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	alertPositionParentWindowScreen
};

resource 'DITL' (kSHLphAppSpaceIsLowNoSave) {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{118 /*was 103*/, 250, 138 /*was 123*/, 310},
		Button {
			enabled,
			"Close"
		},
		/* [2] */
		{118 /*was 103*/, 177, 138 /*was 123*/, 237},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		},
		/* [4] */
		{10, 70, 108 /*was 93*/, 310},
		StaticText {
			disabled,
			"This document is almost out of memory."
			" We recommend that you close it immediately or"
			" close other windows. Use the Document Info command"
			" to increase the amount of memory reserved for it."
		}
	}
};


//-------------------------------------------------------------------------------------
// Misc. Shell Strings
//-------------------------------------------------------------------------------------


resource 'STR#' (kSHLStrsID, purgeable) {
	{
		 "" /* Unused slot. May be reused. See ShellDef.h */
/*		;"Close"				*/
/*		;"Close Document"		*/
		;"Part Info"
		
		;"Save a copy as:"
		
		;"Exception: "
		
		;"Save as:"
		
	}
};

resource 'STR ' (kSHLCopyDefaultStrID, purgeable) {
	"^0 copy"
};

resource 'STR ' (kSHLPreODDocNameStrID, purgeable) {
	"^0 OD"
};



//-------------------------------------------------------------------------------------
// OpenDoc About... box
//-------------------------------------------------------------------------------------



resource 'PICT' (kSHLAboutBoxDlogID) {
	7600,
	{124, 50, 298, 270},
	$"0011 02FF 0C00 FFFF FFFF 0032 0000 007C"
	$"0000 010E 0000 012A 0000 0000 0000 001E"
	$"0001 000A 007C 0032 012A 010E 0099 80DC"
	$"007C 0032 012A 010E 0000 0000 0000 0000"
	$"0048 0000 0048 0000 0000 0008 0001 0008"
	$"0000 0000 00C6 5D2C 0000 0000 0000 05B9"
	$"0000 00FF 0000 FFFF FFFF FFFF 0001 FFFF"
	$"FFFF CCCC 0002 FFFF FFFF 9999 0003 FFFF"
	$"FFFF 6666 0004 FFFF FFFF 3333 0005 FFFF"
	$"FFFF 0000 0006 FFFF CCCC FFFF 0007 FFFF"
	$"CCCC CCCC 0008 FFFF CCCC 9999 0009 FFFF"
	$"CCCC 6666 000A FFFF CCCC 3333 000B FFFF"
	$"CCCC 0000 000C FFFF 9999 FFFF 000D FFFF"
	$"9999 CCCC 000E FFFF 9999 9999 000F FFFF"
	$"9999 6666 0010 FFFF 9999 3333 0011 FFFF"
	$"9999 0000 0012 FFFF 6666 FFFF 0013 FFFF"
	$"6666 CCCC 0014 FFFF 6666 9999 0015 FFFF"
	$"6666 6666 0016 FFFF 6666 3333 0017 FFFF"
	$"6666 0000 0018 FFFF 3333 FFFF 0019 FFFF"
	$"3333 CCCC 001A FFFF 3333 9999 001B FFFF"
	$"3333 6666 001C FFFF 3333 3333 001D FFFF"
	$"3333 0000 001E FFFF 0000 FFFF 001F FFFF"
	$"0000 CCCC 0020 FFFF 0000 9999 0021 FFFF"
	$"0000 6666 0022 FFFF 0000 3333 0023 FFFF"
	$"0000 0000 0024 CCCC FFFF FFFF 0025 CCCC"
	$"FFFF CCCC 0026 CCCC FFFF 9999 0027 CCCC"
	$"FFFF 6666 0028 CCCC FFFF 3333 0029 CCCC"
	$"FFFF 0000 002A CCCC CCCC FFFF 002B CCCC"
	$"CCCC CCCC 002C CCCC CCCC 9999 002D CCCC"
	$"CCCC 6666 002E CCCC CCCC 3333 002F CCCC"
	$"CCCC 0000 0030 CCCC 9999 FFFF 0031 CCCC"
	$"9999 CCCC 0032 CCCC 9999 9999 0033 CCCC"
	$"9999 6666 0034 CCCC 9999 3333 0035 CCCC"
	$"9999 0000 0036 CCCC 6666 FFFF 0037 CCCC"
	$"6666 CCCC 0038 CCCC 6666 9999 0039 CCCC"
	$"6666 6666 003A CCCC 6666 3333 003B CCCC"
	$"6666 0000 003C CCCC 3333 FFFF 003D CCCC"
	$"3333 CCCC 003E CCCC 3333 9999 003F CCCC"
	$"3333 6666 0040 CCCC 3333 3333 0041 CCCC"
	$"3333 0000 0042 CCCC 0000 FFFF 0043 CCCC"
	$"0000 CCCC 0044 CCCC 0000 9999 0045 CCCC"
	$"0000 6666 0046 CCCC 0000 3333 0047 CCCC"
	$"0000 0000 0048 9999 FFFF FFFF 0049 9999"
	$"FFFF CCCC 004A 9999 FFFF 9999 004B 9999"
	$"FFFF 6666 004C 9999 FFFF 3333 004D 9999"
	$"FFFF 0000 004E 9999 CCCC FFFF 004F 9999"
	$"CCCC CCCC 0050 9999 CCCC 9999 0051 9999"
	$"CCCC 6666 0052 9999 CCCC 3333 0053 9999"
	$"CCCC 0000 0054 9999 9999 FFFF 0055 9999"
	$"9999 CCCC 0056 9999 9999 9999 0057 9999"
	$"9999 6666 0058 9999 9999 3333 0059 9999"
	$"9999 0000 005A 9999 6666 FFFF 005B 9999"
	$"6666 CCCC 005C 9999 6666 9999 005D 9999"
	$"6666 6666 005E 9999 6666 3333 005F 9999"
	$"6666 0000 0060 9999 3333 FFFF 0061 9999"
	$"3333 CCCC 0062 9999 3333 9999 0063 9999"
	$"3333 6666 0064 9999 3333 3333 0065 9999"
	$"3333 0000 0066 9999 0000 FFFF 0067 9999"
	$"0000 CCCC 0068 9999 0000 9999 0069 9999"
	$"0000 6666 006A 9999 0000 3333 006B 9999"
	$"0000 0000 006C 6666 FFFF FFFF 006D 6666"
	$"FFFF CCCC 006E 6666 FFFF 9999 006F 6666"
	$"FFFF 6666 0070 6666 FFFF 3333 0071 6666"
	$"FFFF 0000 0072 6666 CCCC FFFF 0073 6666"
	$"CCCC CCCC 0074 6666 CCCC 9999 0075 6666"
	$"CCCC 6666 0076 6666 CCCC 3333 0077 6666"
	$"CCCC 0000 0078 6666 9999 FFFF 0079 6666"
	$"9999 CCCC 007A 6666 9999 9999 007B 6666"
	$"9999 6666 007C 6666 9999 3333 007D 6666"
	$"9999 0000 007E 6666 6666 FFFF 007F 6666"
	$"6666 CCCC 0080 6666 6666 9999 0081 6666"
	$"6666 6666 0082 6666 6666 3333 0083 6666"
	$"6666 0000 0084 6666 3333 FFFF 0085 6666"
	$"3333 CCCC 0086 6666 3333 9999 0087 6666"
	$"3333 6666 0088 6666 3333 3333 0089 6666"
	$"3333 0000 008A 6666 0000 FFFF 008B 6666"
	$"0000 CCCC 008C 6666 0000 9999 008D 6666"
	$"0000 6666 008E 6666 0000 3333 008F 6666"
	$"0000 0000 0090 3333 FFFF FFFF 0091 3333"
	$"FFFF CCCC 0092 3333 FFFF 9999 0093 3333"
	$"FFFF 6666 0094 3333 FFFF 3333 0095 3333"
	$"FFFF 0000 0096 3333 CCCC FFFF 0097 3333"
	$"CCCC CCCC 0098 3333 CCCC 9999 0099 3333"
	$"CCCC 6666 009A 3333 CCCC 3333 009B 3333"
	$"CCCC 0000 009C 3333 9999 FFFF 009D 3333"
	$"9999 CCCC 009E 3333 9999 9999 009F 3333"
	$"9999 6666 00A0 3333 9999 3333 00A1 3333"
	$"9999 0000 00A2 3333 6666 FFFF 00A3 3333"
	$"6666 CCCC 00A4 3333 6666 9999 00A5 3333"
	$"6666 6666 00A6 3333 6666 3333 00A7 3333"
	$"6666 0000 00A8 3333 3333 FFFF 00A9 3333"
	$"3333 CCCC 00AA 3333 3333 9999 00AB 3333"
	$"3333 6666 00AC 3333 3333 3333 00AD 3333"
	$"3333 0000 00AE 3333 0000 FFFF 00AF 3333"
	$"0000 CCCC 00B0 3333 0000 9999 00B1 3333"
	$"0000 6666 00B2 3333 0000 3333 00B3 3333"
	$"0000 0000 00B4 0000 FFFF FFFF 00B5 0000"
	$"FFFF CCCC 00B6 0000 FFFF 9999 00B7 0000"
	$"FFFF 6666 00B8 0000 FFFF 3333 00B9 0000"
	$"FFFF 0000 00BA 0000 CCCC FFFF 00BB 0000"
	$"CCCC CCCC 00BC 0000 CCCC 9999 00BD 0000"
	$"CCCC 6666 00BE 0000 CCCC 3333 00BF 0000"
	$"CCCC 0000 00C0 0000 9999 FFFF 00C1 0000"
	$"9999 CCCC 00C2 0000 9999 9999 00C3 0000"
	$"9999 6666 00C4 0000 9999 3333 00C5 0000"
	$"9999 0000 00C6 0000 6666 FFFF 00C7 0000"
	$"6666 CCCC 00C8 0000 6666 9999 00C9 0000"
	$"6666 6666 00CA 0000 6666 0000 00CB 0000"
	$"6666 3333 00CC 0000 3333 FFFF 00CD 0000"
	$"3333 CCCC 00CE 0000 3333 9999 00CF 0000"
	$"3333 6666 00D0 0000 3333 3333 00D1 0000"
	$"3333 0000 00D2 0000 0000 FFFF 00D3 0000"
	$"0000 CCCC 00D4 0000 0000 9999 00D5 0000"
	$"0000 6666 00D6 0000 0000 3333 00D7 EEEE"
	$"0000 0000 00D8 DDDD 0000 0000 00D9 BBBB"
	$"0000 0000 00DA AAAA 0000 0000 00DB 8888"
	$"0000 0000 00DC 7777 0000 0000 00DD 5555"
	$"0000 0000 00DE 4444 0000 0000 00DF 2222"
	$"0000 0000 00E0 1111 0000 0000 00E1 0000"
	$"EEEE 0000 00E2 0000 DDDD 0000 00E3 0000"
	$"BBBB 0000 00E4 0000 AAAA 0000 00E5 0000"
	$"8888 0000 00E6 0000 7777 0000 00E7 0000"
	$"5555 0000 00E8 0000 4444 0000 00E9 0000"
	$"2222 0000 00EA 0000 1111 0000 00EB 0000"
	$"0000 EEEE 00EC 0000 0000 DDDD 00ED 0000"
	$"0000 BBBB 00EE 0000 0000 AAAA 00EF 0000"
	$"0000 8888 00F0 0000 0000 7777 00F1 0000"
	$"0000 5555 00F2 0000 0000 4444 00F3 0000"
	$"0000 2222 00F4 0000 0000 1111 00F5 EEEE"
	$"EEEE EEEE 00F6 DDDD DDDD DDDD 00F7 BBBB"
	$"BBBB BBBB 00F8 AAAA AAAA AAAA 00F9 8888"
	$"8888 8888 00FA 7777 7777 7777 00FB 5555"
	$"5555 5555 00FC 4444 4444 4444 00FD 2222"
	$"2222 2222 00FE 1111 1111 1111 00FF 0000"
	$"0000 0000 007C 0032 012A 010E 007C 0032"
	$"012A 010E 0000 077C 007C 0032 012A 010E"
	$"007C 0070 0081 7FFF 007D 006A 0070 0081"
	$"0087 7FFF 007E 0066 006A 0087 008B 7FFF"
	$"007F 0063 0066 008B 008E 7FFF 0080 0060"
	$"0063 008E 0091 7FFF 0081 005D 0060 0091"
	$"0094 7FFF 0082 005B 005D 0094 0096 7FFF"
	$"0083 0059 005B 0096 0098 7FFF 0084 0057"
	$"0059 0098 009A 7FFF 0085 0055 0057 009A"
	$"009C 7FFF 0086 0053 0055 009C 009E 7FFF"
	$"0087 0052 0053 009E 009F 7FFF 0088 0050"
	$"0052 009F 00A1 7FFF 0089 004F 0050 00A1"
	$"00A2 7FFF 008A 004E 004F 00A2 00A3 7FFF"
	$"008B 004C 004E 00A3 00A5 7FFF 008C 004B"
	$"004C 00A5 00A6 7FFF 008D 004A 004B 00A6"
	$"00A7 7FFF 008E 0049 004A 00A7 00A8 7FFF"
	$"008F 0048 0049 00A8 00A9 7FFF 0090 0047"
	$"0048 00A9 00AA 7FFF 0091 0046 0047 00AA"
	$"00AB 7FFF 0092 0045 0046 00AB 00AC 7FFF"
	$"0093 0044 0045 00AC 00AD 7FFF 0094 0043"
	$"0044 00AD 00AE 7FFF 0095 0042 0043 00AE"
	$"00AF 7FFF 0096 0041 0042 00AF 00B0 7FFF"
	$"0098 0040 0041 00B0 00B1 7FFF 0099 003F"
	$"0040 00B1 00B2 7FFF 009A 003E 003F 00B2"
	$"00B3 7FFF 009C 003D 003E 00B3 00B4 7FFF"
	$"009D 003C 003D 00B4 00B5 7FFF 009F 003B"
	$"003C 00B5 00B6 7FFF 00A1 003A 003B 00B6"
	$"00B7 7FFF 00A3 0039 003A 00B7 00B8 7FFF"
	$"00A5 0038 0039 00B8 00B9 7FFF 00A7 0037"
	$"0038 00B9 00BA 7FFF 00AA 0036 0037 00BA"
	$"00BB 7FFF 00AD 0035 0036 00BB 00BC 7FFF"
	$"00B0 0034 0035 00BC 00BD 7FFF 00B4 0033"
	$"0034 00BD 00BE 7FFF 00BA 0032 0033 00BE"
	$"00BF 7FFF 00CB 0032 0033 00BE 00BF 7FFF"
	$"00D1 0033 0034 00BD 00BE 7FFF 00D5 0034"
	$"0035 00BC 00BD 7FFF 00D8 0035 0036 00BB"
	$"00BC 7FFF 00DB 0036 0037 00BA 00BB 7FFF"
	$"00DE 0037 0038 00B9 00BA 7FFF 00E0 0038"
	$"0039 00B8 00B9 7FFF 00E2 0039 003A 00B7"
	$"00B8 7FFF 00E4 003A 003B 00B6 00B7 7FFF"
	$"00E6 003B 003C 00B5 00B6 7FFF 00E8 003C"
	$"003D 00B4 00B5 7FFF 00E9 003D 003E 00B3"
	$"00B4 7FFF 00EB 003E 003F 00B2 00B3 7FFF"
	$"00EC 003F 0040 00B1 00B2 7FFF 00ED 0040"
	$"0041 00B0 00B1 7FFF 00EF 0041 0042 00AF"
	$"00B0 7FFF 00F0 0042 0043 00AE 00AF 7FFF"
	$"00F1 0043 0044 00AD 00AE 7FFF 00F2 0044"
	$"0045 00AC 00AD 7FFF 00F3 0045 0046 00AB"
	$"00AC 7FFF 00F4 0046 0047 00AA 00AB 7FFF"
	$"00F5 0047 0048 00A9 00AA 7FFF 00F6 0048"
	$"0049 00A8 00A9 7FFF 00F7 0049 004A 00A7"
	$"00A8 7FFF 00F8 004A 004B 00A6 00A7 7FFF"
	$"00F9 004B 004C 00A5 00A6 7FFF 00FA 004C"
	$"004E 00A3 00A5 7FFF 00FB 004E 004F 00A2"
	$"00A3 7FFF 00FC 004F 0050 00A1 00A2 7FFF"
	$"00FD 0050 0052 009F 00A1 7FFF 00FE 0052"
	$"0053 009E 009F 7FFF 00FF 0053 0055 009C"
	$"009E 7FFF 0100 0055 0057 009A 009C 7FFF"
	$"0101 0057 0059 0098 009A 7FFF 0102 0059"
	$"005B 0096 0098 7FFF 0103 005B 005D 0094"
	$"0096 7FFF 0104 005D 0060 0091 0094 7FFF"
	$"0105 0060 0063 008E 0091 7FFF 0106 0063"
	$"0066 008B 008E 7FFF 0107 0066 006A 0087"
	$"008B 7FFF 0108 006A 0070 0081 0087 7FFF"
	$"0109 0049 0051 0070 0081 00A6 00B6 00F3"
	$"0104 0109 010E 7FFF 010A 0046 0049 0051"
	$"0053 00B6 00B9 0104 0105 0108 0109 7FFF"
	$"010B 0044 0046 0053 0055 00B9 00BA 7FFF"
	$"010C 0043 0044 0055 0056 00BA 00BB 00F3"
	$"00F6 00FA 00FF 0107 0108 7FFF 010D 0042"
	$"0043 0056 0057 00A5 00A6 00BB 00BC 00FE"
	$"00FF 0102 0103 010D 010E 7FFF 010E 0041"
	$"0042 0106 0107 7FFF 010F 0040 0041 0057"
	$"0058 00BC 00BD 7FFF 0110 005D 0063 0066"
	$"006C 007B 0082 008E 0094 0097 009D 00C7"
	$"00CF 00DF 00E7 0105 0106 7FFF 0111 003F"
	$"0040 005C 005D 0064 0066 006C 006E 0078"
	$"007B 0082 0084 008D 008E 0093 0094 0095"
	$"0097 009D 009E 00C4 00C7 00CF 00D1 00DC"
	$"00DF 00E7 00E9 00F5 00F6 00F9 00FA 00FD"
	$"00FE 0101 0102 0107 0108 010C 010D 7FFF"
	$"0112 0063 0064 006E 006F 0076 0078 0084"
	$"0085 0094 0095 009E 009F 00A4 00A5 00C2"
	$"00C4 00D1 00D2 00DA 00DC 00E9 00EA 7FFF"
	$"0113 003E 003F 006F 0070 0075 0076 0085"
	$"0086 0093 0094 00C1 00C2 00D2 00D3 00D9"
	$"00DA 00EA 00EB 0106 0107 7FFF 0114 0074"
	$"0075 009F 00A0 00C0 00C1 00D3 00D4 00D8"
	$"00D9 7FFF 0115 00DF 00E4 7FFF 0116 0057"
	$"0058 005B 005C 0073 0074 0086 0087 008C"
	$"008D 0095 0098 009F 00A0 00BC 00BD 00BF"
	$"00C0 00D7 00D8 00DE 00DF 00F5 00F9 00FD"
	$"0101 0103 0106 0108 010C 7FFF 0117 003D"
	$"003E 0094 0095 00A3 00A4 00BE 00BF 00D6"
	$"00D7 00E4 00EB 7FFF 0118 0072 0073 0093"
	$"0094 00DD 00DE 7FFF 0119 0056 0057 7FFF"
	$"011A 00BB 00BC 7FFF 011B 0055 0056 005A"
	$"005B 006F 0070 0086 0087 008B 008C 0092"
	$"0093 009E 009F 00D3 00D4 00E3 00EA 7FFF"
	$"011C 003D 003E 0078 0086 0097 0098 00A2"
	$"00A3 00BA 00BB 7FFF 011D 0054 0055 006E"
	$"006F 0078 0079 00B9 00BA 00D2 00D3 00DD"
	$"00DE 00E2 00E3 7FFF 011E 003E 003F 0053"
	$"0054 0079 007A 007E 0086 00B8 00B9 00DE"
	$"00E2 00E9 00EA 7FFF 011F 0052 0053 006D"
	$"006E 0072 0073 007A 007E 0085 0086 00B7"
	$"00B8 00BE 00BF 00D1 00D2 00D6 00D7 00E8"
	$"00E9 7FFF 0120 003F 0040 0051 0052 0059"
	$"005A 006C 006D 0073 0074 0084 0085 008A"
	$"008B 0091 0092 009D 009E 00B6 00B7 00BF"
	$"00C0 00CF 00D1 00D7 00D8 00E7 00E8 7FFF"
	$"0121 0040 0042 004F 0051 0060 0061 006A"
	$"006C 0074 0075 0082 0084 0096 0097 00A1"
	$"00A2 00B5 00B6 00C0 00C1 00CE 00CF 00D8"
	$"00D9 00E5 00E7 7FFF 0122 0042 0044 004C"
	$"004F 0061 0063 0068 006A 0075 0077 007F"
	$"0082 00B2 00B5 00C1 00C3 00CB 00CE 00D9"
	$"00DB 00E3 00E5 7FFF 0123 0044 004C 0063"
	$"0068 0077 007F 008A 0091 0096 009D 00A1"
	$"00B2 00C3 00CB 00DB 00E3 7FFF 0125 0058"
	$"0059 005F 0060 7FFF 012A 0058 005F 7FFF"
	$"7FFF 08C3 00F0 FA81 00F4 0008 C900 E4FA"
	$"8100 FA00 08CD 00DC FA81 00FE 0006 D000"
	$"D6FA 8100 06D3 00D0 FA84 0006 D600 CAFA"
	$"8700 06D8 00C6 FA89 0006 DA00 C2FA 8B00"
	$"06DC 00BE FA8D 000A DE00 E3FA F3F8 E6FA"
	$"8F00 0DE0 00E3 FA01 0B0B EDF8 EAFA 9100"
	$"0CE1 00E8 FAF9 0BE9 F8ED FA92 000C E300"
	$"EAFA F50B E6F8 EEFA 9400 0CE4 00EC FAF2"
	$"0BE4 F8EF FA95 000C E500 EDFA F00B E1F8"
	$"F1FA 9600 0CE7 00EE FAED 0BDF F8F1 FA98"
	$"000C E800 EFFA EB0B DDF8 F2FA 9900 0CE9"
	$"00F0 FAE9 0BDB F8F3 FA9A 000C EA00 F0FA"
	$"E80B DAF8 F3FA 9B00 10EB 00F1 FAE6 0BF7"
	$"F8FE F6E5 F8F4 FA9C 0010 EC00 F2FA E40B"
	$"F7F8 F8F6 EAF8 F4FA 9D00 10ED 00F2 FAE3"
	$"0BF7 F8F5 F6EB F8F5 FA9E 0010 EE00 F2FA"
	$"E20B F7F8 F2F6 EDF8 F5FA 9F00 10EF 00F3"
	$"FAE0 0BF7 F8F0 F6EE F8F5 FAA0 0010 F000"
	$"F3FA DF0B F7F8 EEF6 EFF8 F5FA A100 10F1"
	$"00F3 FADE 0BF7 F8EC F6F0 F8F5 FAA2 0010"
	$"F200 F3FA DD0B F7F8 EAF6 F1F8 F5FA A300"
	$"10F2 00F4 FADC 0BF7 F8E8 F6F2 F8F6 FAA3"
	$"0010 F300 F4FA DB0B F7F8 E7F6 F2F8 F6FA"
	$"A400 10F4 00F4 FADA 0BF7 F8E5 F6F3 F8F6"
	$"FAA5 0010 F500 F4FA D90B F7F8 E4F6 F3F8"
	$"F6FA A600 14F5 00F5 FAEC 0BF8 05F6 0BF7"
	$"F8E3 F6F3 F8F7 FAA6 0014 F600 F5FA EE0B"
	$"F205 F90B F7F8 E2F6 F3F8 F7FA A700 14F7"
	$"00F4 FAF1 0BEC 05FC 0BF7 F8E1 F6F3 F8F7"
	$"FAA8 0014 F700 F5FA F10B EA05 FD0B F7F8"
	$"E0F6 F4F8 F7FA A800 15F8 00F5 FAF2 0BE6"
	$"0501 0B0B F7F8 DFF6 F4F8 F7FA A900 14F8"
	$"00F6 FAF2 0BE4 0500 0BF7 F8DE F6F4 F8F8"
	$"FAA9 0012 F900 F5FA F30B E205 F7F8 DDF6"
	$"F5F8 F7FA AA00 12F9 00F6 FAF3 0BE1 05F7"
	$"F8DC F6F5 F8F8 FAAA 0012 FA00 F5FA F40B"
	$"E005 F7F8 DBF6 F5F8 F8FA AB00 14FA 00F6"
	$"FAF3 0BE0 05F7 F8DB F6F5 F800 F6F9 FAAB"
	$"0012 FB00 F6FA F30B DF05 F7F8 DAF6 F5F8"
	$"F8FA AC00 14FB 00F6 FAF4 0BDE 05F7 F8D9"
	$"F6F6 F800 F6F9 FAAC 0012 FC00 F6FA F30B"
	$"DE05 F7F8 D9F6 F5F8 F8FA AD00 14FC 00F6"
	$"FAF3 0BDE 05F7 F8D8 2BF6 F800 2BF9 FAAD"
	$"0012 FC00 F7FA F30B DD05 F7F8 D72B F6F8"
	$"F9FA AD00 14FD 00F6 FAF3 0BDD 05F7 F8D7"
	$"2BF6 F800 2BF9 FAAE 0012 FD00 F6FA F30B"
	$"DD05 F7F8 D62B F6F8 F9FA AE00 12FD 00F7"
	$"FAF3 0BDC 05F7 F8D6 2BF6 F8F9 FAAE 0012"
	$"FE00 F6FA F30B DC05 F7F8 D52B F7F8 F8FA"
	$"AF00 12FE 00F7 FAF2 0BDC 05F7 F8D5 2BF6"
	$"F8F9 FAAF 0012 FE00 F7FA F20B DC05 F7F8"
	$"D52B F6F8 F9FA AF00 1501 0000 F6FA F20B"
	$"DC05 F7F8 D42B F7F8 002B F9FA B000 1301"
	$"0000 F7FA F10B DC05 F7F8 D42B F6F8 F9FA"
	$"B000 1301 0000 F7FA F10B DC05 F7F8 D32B"
	$"F7F8 F9FA B000 1301 0000 F7FA F10B DC05"
	$"F7F8 D32B F7F8 F9FA B000 1400 00F6 FAF1"
	$"0BDC 05F7 F8D3 2BF7 F800 2BF9 FAB1 0012"
	$"0000 F7FA F00B DC05 F7F8 D32B F6F8 F9FA"
	$"B100 1200 00F7 FAEF 0BDD 05F7 F8D2 2BF7"
	$"F8F9 FAB1 0012 0000 F7FA EF0B DD05 F7F8"
	$"D22B F7F8 F9FA B100 1200 00F7 FAEF 0BDD"
	$"05F7 F8D2 F7F7 F8F9 FAB1 0014 0000 F7FA"
	$"EE0B DE05 F7F8 D2F7 F7F8 00F7 FAFA B100"
	$"10F6 FAEE 0BDE 05F7 F8D2 F7F6 F8F9 FAB2"
	$"0010 F7FA EC0B DF05 F7F8 D1F7 F7F8 F9FA"
	$"B200 10F7 FAEC 0BDF 05F7 F8D1 F7F7 F8F9"
	$"FAB2 0010 F7FA EB0B E005 F7F8 D1F7 F7F8"
	$"F9FA B200 10F7 FAEA 0BE1 05F7 F8D1 F7F7"
	$"F8F9 FAB2 0010 F7FA E90B E205 F7F8 D1F7"
	$"F7F8 F9FA B200 12F7 FAE8 0BE4 0500 0BF7"
	$"F8D1 F7F7 F8F9 FAB2 0013 F7FA E70B E605"
	$"010B 0BF7 F8D1 F7F7 F8F9 FAB2 0012 F7FA"
	$"E60B E905 FD0B F7F8 D1F7 F7F8 F9FA B200"
	$"0EF7 FAE4 0BEC 05FC 0BBD F8F9 FAB2 000E"
	$"F7FA E20B F105 F90B BDF8 F9FA B200 0EF7"
	$"FADF 0BF7 05F6 0BBD F8F9 FAB2 000A F7FA"
	$"CA0B BDF8 F9FA B200 0AF7 FACA 0BBD F8F9"
	$"FAB2 000A F7FA CA0B BDF8 F9FA B200 0CF7"
	$"FA00 11CB 0BBD F8F9 FAB2 000A F6FA CB0B"
	$"BDF8 F9FA B200 0C00 00F7 FACB 0BBE F8F9"
	$"FAB1 000C 0000 F7FA CB0B BEF8 F9FA B100"
	$"1200 00F7 FA00 11CC 0BF7 F8D2 56F7 F8F9"
	$"FAB1 0012 0000 F7FA 0011 CC0B F7F8 D356"
	$"F6F8 F9FA B100 1200 00F7 FA00 11CC 0BF7"
	$"F8D3 56F7 F8F8 FAB1 0012 0000 F6FA 0011"
	$"CD0B F7F8 D356 F7F8 F8FA B100 1301 0000"
	$"F7FA 0011 CD0B F7F8 D356 F7F8 F9FA B000"
	$"1401 0000 F7FA 0111 11CE 0BF7 F8D4 56F6"
	$"F8F9 FAB0 0014 0100 00F7 FA01 1111 CE0B"
	$"F7F8 D456 F7F8 F8FA B000 1401 0000 F6FA"
	$"0111 11CF 0BF7 F8D5 56F6 F8F8 FAB0 0013"
	$"FE00 F7FA 0111 11CF 0BF7 F8D5 56F6 F8F9"
	$"FAAF 0012 FE00 F7FA FE11 D00B F7F8 D5F9"
	$"F7F8 F8FA AF00 13FE 00F6 FA01 1111 D00B"
	$"F7F8 D6F9 F6F8 F8FA AF00 12FD 00F7 FAFE"
	$"11D1 0BF7 F8D6 F9F6 F8F9 FAAE 0013 FD00"
	$"F6FA 0111 11D1 0BF7 F8D7 F9F6 F8F8 FAAE"
	$"0012 FD00 F6FA FE11 D20B F7F8 D7F9 F6F8"
	$"F8FA AE00 12FC 00F7 FAFD 11D3 0BF7 F8D8"
	$"F9F6 F8F8 FAAD 0012 FC00 F6FA FE11 D30B"
	$"F7F8 D9F9 F5F8 F8FA AD00 12FC 00F6 FAFD"
	$"11D4 0BF7 F8D9 FAF6 F8F7 FAAD 0012 FB00"
	$"F6FA FD11 D50B F7F8 DAFA F5F8 F8FA AC00"
	$"12FB 00F6 FAFC 11D6 0BF7 F8DB FAF5 F8F7"
	$"FAAC 0012 FA00 F6FA FC11 D70B F7F8 DBFA"
	$"F5F8 F8FA AB00 12FA 00F5 FAFD 11D7 0BF7"
	$"F8DC FAF5 F8F7 FAAB 0012 F900 F6FA FC11"
	$"D80B F7F8 DDFA F5F8 F7FA AA00 12F9 00F5"
	$"FAFC 11D9 0BF7 F8DE FAF4 F8F7 FAAA 0012"
	$"F800 F6FA FB11 DA0B F7F8 DF81 F4F8 F7FA"
	$"A900 12F8 00F5 FAFA 11DC 0BF7 F8E0 81F4"
	$"F8F6 FAA9 0012 F700 F5FA FA11 DD0B F7F8"
	$"E181 F3F8 F7FA A800 12F7 00F4 FAFA 11DE"
	$"0BF7 F8E2 81F3 F8F6 FAA8 0012 F600 F5FA"
	$"F911 DF0B F7F8 E381 F3F8 F6FA A700 12F5"
	$"00F5 FAF8 11E1 0BF7 F8E4 81F3 F8F6 FAA6"
	$"0012 F500 F4FA F811 E20B F7F8 E581 F3F8"
	$"F5FA A600 12F4 00F4 FAF7 11E4 0BF7 F8E7"
	$"FBF2 F8F5 FAA5 0012 F300 F4FA F611 E60B"
	$"F7F8 E8FB F2F8 F5FA A400 12F2 00F4 FAF5"
	$"11E8 0BF7 F8EA FBF1 F8F5 FAA3 0012 F200"
	$"F3FA F411 EA0B F7F8 ECFB F0F8 F4FA A300"
	$"12F1 00F3 FAF3 11EC 0BF7 F8EE FBEF F8F4"
	$"FAA2 0012 F000 F3FA F111 EF0B F7F8 F0FB"
	$"EEF8 F4FA A100 12EF 00F3 FAEE 11F3 0BF7"
	$"F8F2 FCED F8F4 FAA0 0012 EE00 F2FA EB11"
	$"F80B F7F8 F5FC EBF8 F4FA 9F00 10ED 00F2"
	$"FAE3 11F7 F8F8 FCEA F8F3 FA9E 0010 EC00"
	$"F2FA E411 F7F8 FEFC E5F8 F3FA 9D00 0CEB"
	$"00F1 FAE6 11DA F8F2 FA9C 000C EA00 F0FA"
	$"E811 DBF8 F2FA 9B00 0CE9 00F0 FAE9 11DD"
	$"F8F1 FA9A 000C E800 EFFA EB11 DFF8 F0FA"
	$"9900 0CE7 00EE FAED 11E1 F8EF FA98 000C"
	$"E500 EDFA F011 E4F8 EEFA 9600 0CE4 00EC"
	$"FAF2 11E6 F8ED FA95 000C E300 EAFA F511"
	$"E9F8 EBFA 9400 0CE1 00E8 FAF9 11ED F8E9"
	$"FA92 000D E000 E3FA 0111 11F3 F8E4 FA91"
	$"0006 DE00 BAFA 8F00 06DC 00BE FA8D 0006"
	$"DA00 C2FA 8B00 06D8 00C6 FA89 0006 D600"
	$"CAFA 8700 06D3 00D0 FA84 0006 D000 D6FA"
	$"8100 08CD 00DC FA81 00FE 0008 C900 E4FA"
	$"8100 FA00 08C3 00F0 FA81 00F4 0010 EA00"
	$"F9FF AC00 F1FF C400 F0FF FC00 FCFF 10ED"
	$"00F4 FFAE 00EE FFC7 00EF FFFE 00FB FF10"
	$"EF00 F0FF B000 EDFF C800 EFFF FE00 FBFF"
	$"15F0 00EE FFB1 00EC FFC6 00FD FFFC 00FB"
	$"FF01 0000 FAFF 1AF1 00EC FFB3 00EA FFC7"
	$"00FD FFFD 00FD FF04 00FF FF00 00FB FF00"
	$"0019 F200 EBFF B300 EAFF C700 FDFF FD00"
	$"FDFF 0300 FFFF 00FA FF00 0024 F300 F7FF"
	$"FD00 F7FF B400 F9FF FB00 F7FF C800 FDFF"
	$"FD00 FDFF 0600 FFFF 00FF FF00 FDFF 0000"
	$"3EF3 00F9 FFFA 00F8 FFFC 00FB FFFE 00FB"
	$"FFF2 00FA FFF5 00FB FFFE 00FB FFF9 00F9"
	$"FFFA 00F8 FFF7 00F9 FFF1 00F9 FFF2 00FD"
	$"FFFD 00FD FF00 00FC FF00 00FD FF00 0040"
	$"F400 F8FF F900 F9FF FD00 FAFF 0000 F7FF"
	$"F700 F5FF F800 FBFF 0100 00F8 FFFA 00F9"
	$"FFF9 00F9 FFFA 00F4 FFF6 00F4 FFF5 00FD"
	$"FFFD 00FD FF01 0000 FDFF 0000 FDFF FF00"
	$"3BF4 00F9 FFF8 00F9 FFFD 00EE FFFA 00F2"
	$"FFF9 00FB FF00 00F6 FFFC 00F9 FFF8 00F9"
	$"FFFC 00F1 FFF9 00F1 FFF6 00FD FFFD 00FD"
	$"FF01 0000 FDFF 0000 FDFF FF00 38F5 00F9"
	$"FFF7 00F9 FFFD 00ED FFFC 00F0 FFFA 00EF"
	$"FFFC 00F9 FFF8 00F9 FFFD 00EF FFFB 00EF"
	$"FFF7 00FD FFFD 00FD FF01 0000 FEFF 0100"
	$"00FD FFFF 003C F500 F9FF F700 F9FF FD00"
	$"EDFF FD00 F9FF FE00 FAFF FA00 EEFF FD00"
	$"F9FF F800 F9FF FE00 EDFF FD00 EEFF F700"
	$"FDFF FD00 FDFF 0100 00FE FF01 0000 FDFF"
	$"FF00 48F5 00F9 FFF7 00F9 FFFD 00F9 FFFD"
	$"00F9 FFFD 00FB FFFB 00FB FFFA 00EE FFFD"
	$"00F9 FFF8 00F9 FFFE 00F9 FFFD 00F9 FFFD"
	$"00FA FFFC 00FA FFF7 00FD FFFD 00FD FF01"
	$"0000 FEFF 0100 00FD FFFF 003A F500 F9FF"
	$"F800 F9FF FD00 F9FF FB00 FAFF FE00 FAFF"
	$"FB00 FAFF FC00 F8FF FE00 FAFF FC00 F9FF"
	$"F900 F9FF FE00 FAFF FA00 FAFF FE00 FAFF"
	$"FB00 FAFF DE00 38F6 00F9 FFF7 00F9 FFFD"
	$"00FA FFFA 00FA FFFE 00FB FFFA 00FA FFFC"
	$"00F9 FFFD 00FA FFFD 00F9 FFF8 00F9 FF01"
	$"0000 F9FF FA00 FAFF 0100 00F9 FFD1 0035"
	$"F600 F9FF F700 F9FF FD00 FAFF FA00 FAFF"
	$"0100 00EC FFFC 00FA FFFC 00FA FFFD 00F9"
	$"FFF8 00F9 FF01 0000 FAFF F900 FAFF 0100"
	$"00FA FFD0 0035 F600 F9FF F800 F9FF FC00"
	$"FAFF FA00 FAFF 0100 00EC FFFC 00FA FFFC"
	$"00FA FFFD 00F9 FFF9 00F8 FF01 0000 FAFF"
	$"F900 FAFF 0100 00FA FFD0 0034 F600 F8FF"
	$"F900 F9FF FC00 FAFF FB00 F9FF 0100 00EC"
	$"FFFC 00FA FFFC 00FA FFFD 00F9 FFF9 00F9"
	$"FFFE 00FA FFF9 00FA FF01 0000 FAFF D000"
	$"36F6 00F8 FFFA 00F9 FFFC 00F9 FFFB 00FA"
	$"FFFE 00ED FFFC 00FA FFFB 00FB FFFC 00F9"
	$"FFFA 00F8 FFFE 00FA FFFA 00FA FFFE 00FA"
	$"FFFB 00FA FFDD 0036 F500 F8FF FD00 F7FF"
	$"FC00 F8FF FD00 F9FF FE00 FBFF EE00 FAFF"
	$"FC00 FAFF FD00 F9FF FB00 F7FF FD00 FAFF"
	$"FA00 FAFF FE00 FAFF FB00 FAFF DD00 2AF5"
	$"00EB FFFB 00ED FFFD 00FA FFEF 00FA FFFC"
	$"00FA FFFD 00EA FFFC 00F9 FFFD 00F9 FFFD"
	$"00F9 FFFD 00F9 FFDD 0026 F400 EDFF FA00"
	$"EDFF FD00 F9FF FD00 F9FF FC00 FAFF FC00"
	$"FAFF FD00 EBFF FB00 EDFF FD00 EEFF DC00"
	$"22F4 00EE FFF9 00EE FFFB 00EF FFFB 00FA"
	$"FFFC 00FA FFFD 00EC FFF9 00EF FFFB 00F0"
	$"FFDB 0022 F300 F0FF F900 EEFF F900 F1FF"
	$"FB00 FAFF FB00 FBFF FC00 EDFF F700 F2FF"
	$"F800 F2FF DA00 26F1 00F4 FFF7 00FA FF00"
	$"00F8 FFF6 00F4 FFF9 00FA FFFC 00FA FFFD"
	$"00ED FFF5 00F4 FFF6 00F5 FFD8 0026 EF00"
	$"F9FF F400 FAFF FE00 FCFF F200 F9FF F600"
	$"FAFF FC00 FAFF FD00 F0FF F000 F9FF F100"
	$"F9FF D600 08DA 00FA FF81 00D3 0008 DA00"
	$"FAFF 8100 D300 08DB 00FA FF81 00D2 0008"
	$"DB00 FAFF 8100 D200 08DB 00FA FF81 00D2"
	$"0008 DB00 FAFF 8100 D200 08DB 00FA FF81"
	$"00D2 0000 00FF"
};

resource 'DLOG' (kODSplashDlogID) {
	{29, 48, 332, 345},
	dBoxProc,
	visible,
	goAway,
	0x0,
	kODSplashDlogID,
	"",
	centerMainScreen
};

data 'ictb' (kODSplashDlogID) {
	$"0000 0000"	/* User item is unaffected */
	$"0005 0008"	/* Change font + size of text item */
	
	$"0001"			/* Use application font */
	$"0000"			/* Plain style [ignored] */
	$"000A"			/* 10pt size */
	$"0000"			/* Red component [ignored] */
	$"0000"			/* Blue component [ignored] */
	$"0000"			/* Green component [ignored] */
	$"0000"			/* BG Red component [ignored] */
	$"0000"			/* BG Blue component [ignored] */
	$"0000"			/* BG Green component [ignored] */
	$"0001"			/* srcOr mode [ignored] */
};

include "OpenDocX.res" 'dctb' (131);
include "OpenDocX.res" 'DITL' (131);
include "OpenDocX.res" 'PICT' (131:134);

/*

resource 'DITL' (kSHLAboutBoxDlogID) {
	{	
		{11, 9, 185, 229},
		Picture {
			enabled,
			kSHLAboutBoxDlogID
		}
	}
};

resource 'DLOG' (kSHLAboutBoxDlogID) {
	{0, 0, 195, 240},
	movableDBoxProc,
	visible,
	noGoAway,
	0x0,
	kSHLAboutBoxDlogID,
	"",
	alertPositionParentWindowScreen
};
*/


/********* Stuff for the Debug menu **********/

/*#if ODDebug*/

resource 'ALRT' (980, "Heap Info [debug only]") {
	{40, 40, 201, 412},
	980,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, silent,
		/* [2] */
		OK, visible, silent,
		/* [3] */
		OK, visible, silent,
		/* [4] */
		OK, visible, silent
	},
	alertPositionParentWindowScreen
};


resource 'DITL' (980, "Heap Info [debug only]") {
	{	/* array DITLarray: 7 elements */
		/* [1] */
		{132, 296, 152, 354},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 72, 29, 360},
		StaticText {
			disabled,
			"^0:"
		},
		/* [3] */
		{28, 72, 79, 162},
		StaticText {
			disabled,
			"Bytes used:\nBytes free:\nBlock count:"
		},
		/* [4] */
		{28, 161, 79, 280},
		StaticText {
			disabled,
			"^1"
		},
		/* [5] */
		{89, 72, 108, 360},
		StaticText {
			disabled,
			"^2:"
		},
		/* [6] */
		{107, 72, 158, 162},
		StaticText {
			disabled,
			"Bytes used:\nBytes free:\nLargest free space:"
		},
		/* [7] */
		{107, 161, 158, 280},
		StaticText {
			disabled,
			"^3"
		}
	}
};


resource 'ALRT' (493,
#if ODDEBUG
"Shell Low Memory Error",
#endif
preload) {
	{40, 40, 142, 318},
	493,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	},
	alertPositionMainScreen
};

/* NOTE THAT WE EXPLICITLY USE ICON #0 IN THIS DITL RATHER THAN CALLING
	StopAlert TO INSERT THE ICON. WE DO THIS BECAUSE WE WANT TO MAKE SURE
	THE ICON IS ALWAYS LOADED INTO MEMORY */

resource 'DITL' (493,
#if ODDEBUG
"Shell Low Memory Error",
#endif
preload) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{69, 201, 89, 263},
		Button {
			enabled,
			"Quit"
		},
		/* [2] */
		{10, 72, 58, 265},
		StaticText {
			disabled,
			"Sorry, this document must immediately qu"
			"it due to very low memory."
		},
		/* [3] */
		{11, 20, 43, 52},
		Icon {
			disabled,
			0
		}
	}
};


/*#endif*/
