/*
	File:		OpenDoc.r

	Contains:	Rez file to merge together subsystem resources
				for single-library OpenDoc build

	Owned by:	Jens Alfke

	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 9/21/95	JP		Uncommented Messaging.rsrc
		 <1>	 8/11/95	jpa		first checked in

	To Do:
*/


#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#ifndef __SYSTYPES_R__
	#include "SysTypes.r"
#endif __SYSTYPES_R__

#ifndef __TYPES_R__
	#include "Types.r"
#endif __TYPES_R__

#ifndef __ODTYPES_R__
	#include "ODTypes.r"
#endif __ODTYPES_R__

#ifndef _BENTODEF_
	#include "BentoDef.h"
#endif _BENTODEF_

#include "CodeFragmentTypes.r"


/*	Include subsystem .r files, ignoring all the 'vers' resources.
	Several of these have nothing other than a 'vers', so these are
	commented out to avoid Rez's "no resources copied" warning.
	If any resources are added to these subsystems, uncomment them! */

include "Bento PPC .rsrc"		'nmap';
include "Binding.rsrc"			not 'vers';
include "Core.rsrc"				not 'vers';
/* include "Imaging.rsrc"		not 'vers'; */
include "Layout.rsrc"			not 'vers';
include "Messaging.rsrc"		not 'vers';
/* include "OSL.rsrc"			not 'vers'; */
include "SharedUtils.rsrc"		not 'vers';
include "Shell.rsrc"			not 'vers';
include "Storage.rsrc"			not 'vers';
include "UI.rsrc"				not 'vers';



/*	Here's the huge 'cfrg' resource. This declares:
	(a) "OpenDocLib", which is what the document stub links to.
	(b) All the names of the old libraries, for part compatibility.
	(c) The four Bento class exports. I am not sure if this is really
		necessary, since the classes are initialized when OpenDocLib
		loads, but I've left them in for safety.
*/

#ifdef PowerPC
#define ARCH	kPowerPC
#else
#define ARCH	kMotorola
#endif

resource 'cfrg' (0) {
	{
		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		"OpenDocLib",

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		"OpenDoc Binding",

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		"OpenDoc Core",

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		"OpenDoc Imaging",

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		"OpenDoc Layout",

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		"OpenDoc Messaging",

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		"OpenDoc Shell",

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		"OpenDoc Storage",

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		"OpenDoc User Interface",

		/* Bento fragments: */
		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		"OpenDoc Bento",

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		kODFileContainerID,

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		kODMemoryContainerID,

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		kODEmbeddedContainerID,

		ARCH,
		kFullLib, kNoVersionNum, kNoVersionNum, kDefaultStackSize, kNoAppSubFolder,
		kIsLib, kOnDiskFlat, kZeroOffset, kWholeFork,
		kODUpdateContainerID
	}
};
