/*
	File:		PfTypLs.cpp

	Contains:	Implementation of class ODPlatformTypeList.

	Owned by:	Craig Carper

	Copyright:	 1994 - 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <6>	  9/6/95	VL		1279074: Replaced THROW_IF_NULL with
									ODSetSOMException.
		 <5>	 8/30/95	EL		1279832: Storage THROW_IF_NULL problems.
		 <4>	  8/3/95	RR		#1257260: Collapse B classes. Remove
									somInit methods. Don't call IsInitialized
									or SubclassResponsibility
		 <3>	 5/26/95	VL		1251403: Multithreading naming support.
		 <2>	 4/25/95	CC		1242555: Removed obsolete $5 comment.
									1242620: InitPlatformTypeList: Added
									ODVolatile().  Use plain parent macros.
		 <1>	 6/23/94	CC		first checked in
		 <0>	 6/23/94	SV		SOMverted
		 <2>	  4/7/94	JA		Added missing #include (1147588)
		 <1>	  4/4/94	CC		first checked in
		 <0>	  4/5/94	CC		Code extracted from TranslaM.cpp (1153046)

	To Do:
*/

#define VARIABLE_MACROS

#define ODPlatformTypeList_Class_Source
#include <PfTypLs.xih>

#ifndef SOM_ODPlatformTypeListIterator_xh
#include <PfTLItr.xh>
#endif

#ifndef SOM_ODObject_xh
#include <ODObject.xh>
#endif

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#pragma segment ODPlatformTypeList

//==============================================================================
// Class ODPlatformTypeList
//==============================================================================

//------------------------------------------------------------------------------
// ODPlatformTypeList: somUninit
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK ODPlatformTypeListsomUninit(ODPlatformTypeList *somSelf)
{
    ODPlatformTypeListData *somThis = ODPlatformTypeListGetData(somSelf);
    ODPlatformTypeListMethodDebug("ODPlatformTypeList","somUninit");

	delete _fList;

    parent_somUninit(somSelf);
}

//------------------------------------------------------------------------------
// ODPlatformTypeList: InitPlatformTypeList
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK ODPlatformTypeListInitPlatformTypeList(ODPlatformTypeList *somSelf, Environment *ev,
		ODPlatformTypeList* typeList)
{
    ODPlatformTypeListData *somThis = ODPlatformTypeListGetData(somSelf);
    ODPlatformTypeListMethodDebug("ODPlatformTypeList","InitPlatformTypeList");

	ODPlatformTypeListIterator* iter = kODNULL;
	ODVolatile(iter);

	SOM_TRY

		/* Moved from somInit. SOM itself sets fields to zero
		_fList = kODNULL;
		*/
		somSelf->InitObject(ev);
	
		_fList = new OrderedCollection;
		
		if ( typeList != (ODPlatformTypeList*) kODNULL )
		{
			iter = typeList->CreatePlatformTypeListIterator(ev);
			
			for (ODPlatformType type = iter->First(ev);
				 iter->IsNotComplete(ev);
				 type = iter->Next(ev))
			{
				somSelf->AddLast(ev,type);
			}
		}

	SOM_CATCH_ALL

		ODDeleteObject(_fList);

	SOM_ENDTRY
	
	ODDeleteObject(iter);
}

//------------------------------------------------------------------------------
// ODPlatformTypeList: AddLast
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK ODPlatformTypeListAddLast(ODPlatformTypeList *somSelf, Environment *ev,
		ODPlatformType type)
{
    ODPlatformTypeListData *somThis = ODPlatformTypeListGetData(somSelf);
    ODPlatformTypeListMethodDebug("ODPlatformTypeList","AddLast");

	SOM_CATCH return;

	if ( somSelf->Contains(ev,type) == kODFalse )
		_fList->AddLast((ElementType)type);
}

//------------------------------------------------------------------------------
// ODPlatformTypeList: Remove
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK ODPlatformTypeListRemove(ODPlatformTypeList *somSelf, Environment *ev,
		ODPlatformType type)
{
    ODPlatformTypeListData *somThis = ODPlatformTypeListGetData(somSelf);
    ODPlatformTypeListMethodDebug("ODPlatformTypeList","Remove");

	SOM_CATCH return;

	if ( somSelf->Contains(ev,type) )
		_fList->Remove((ElementType)type);
}

//------------------------------------------------------------------------------
// ODPlatformTypeList: Contains
//------------------------------------------------------------------------------

SOM_Scope ODBoolean  SOMLINK ODPlatformTypeListContains(ODPlatformTypeList *somSelf, Environment *ev,
		ODPlatformType type)
{
    ODPlatformTypeListData *somThis = ODPlatformTypeListGetData(somSelf);
    ODPlatformTypeListMethodDebug("ODPlatformTypeList","Contains");

	SOM_CATCH return kODFalse;

	return _fList->Contains((ElementType)type);
}

//------------------------------------------------------------------------------
// ODPlatformTypeList: Count
//------------------------------------------------------------------------------

SOM_Scope ODULong  SOMLINK ODPlatformTypeListCount(ODPlatformTypeList *somSelf, Environment *ev)
{
    ODPlatformTypeListData *somThis = ODPlatformTypeListGetData(somSelf);
    ODPlatformTypeListMethodDebug("ODPlatformTypeList","Count");

	SOM_CATCH return 0;

	return _fList->Count();
}

//------------------------------------------------------------------------------
// ODPlatformTypeList: CreatePlatformTypeListIterator
//------------------------------------------------------------------------------

SOM_Scope ODPlatformTypeListIterator*  SOMLINK ODPlatformTypeListCreatePlatformTypeListIterator(ODPlatformTypeList *somSelf, Environment *ev)
{
    ODPlatformTypeListData *somThis = ODPlatformTypeListGetData(somSelf);
    ODPlatformTypeListMethodDebug("ODPlatformTypeList","CreatePlatformTypeListIterator");

	SOM_CATCH return (ODPlatformTypeListIterator*) kODNULL;

	ODPlatformTypeListIterator* iter = new ODPlatformTypeListIterator;
	if (iter == kODNULL)
		ODSetSOMException(ev, kODErrOutOfMemory);
	else
		iter->InitPlatformTypeListIterator(ev, somSelf);
	
	return iter;
}

//------------------------------------------------------------------------------
// ODPlatformTypeList: GetImplementation
//------------------------------------------------------------------------------

SOM_Scope OrderedCollection*  SOMLINK ODPlatformTypeListGetImplementation(ODPlatformTypeList *somSelf, Environment *ev)
{
    ODPlatformTypeListData *somThis = ODPlatformTypeListGetData(somSelf);
    ODPlatformTypeListMethodDebug("ODPlatformTypeList","GetImplementation");

    return _fList;
}
