/*
	File:		StorageU.cpp

	Contains:	Implementation of ODStorageUnit

	Owned by:	Vincent Lo

	Copyright:	 1994 - 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<11>	  8/3/95	RR		#1257260: Collapse B classes. Remove
									somInit methods. Don't call IsInitialized
									or SubclassResponsibility
		<10>	 5/26/95	VL		1251403: Multithreading naming support.
		 <9>	 5/19/95	VL		1249626: in length and out byteArray for
									GetValue and GetPromiseValue.
		 <8>	 3/27/95	VL		1231412: Removed CopyTo.
		 <7>	 3/10/95	VL		1226116: Added SetStorageUnitRef.
		 <6>	 1/26/95	VL		#???: SURef is not a 4-byte array.
		 <5>	12/20/94	VL		1195012: Make Storage calls be
									marshallable.
		 <4>	 8/26/94	VL		1183174: Removed CloneTo.
		 <3>	 8/11/94	VL		Added CreateCursor, modified CreateView and
									changed GetStorageUnitRefIterator to
									CreateStorageUnitRefIterator.
		 <2>	  7/5/94	VL		Added SubClassResponsibility.
		 <1>	 6/28/94	VL		first checked in

	To Do:
*/

#define ODStorageUnit_Class_Source
#include <StorageU.xih>

SOM_Scope void  SOMLINK ODStorageUnitInitStorageUnit(ODStorageUnit *somSelf, Environment *ev,
		ODDraft* draft,
		ODStorageUnitID suid)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitInitStorageUnit");
}

SOM_Scope ODDraft*  SOMLINK ODStorageUnitGetDraft(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetDraft");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODNULL;
}

SOM_Scope ODSession*  SOMLINK ODStorageUnitGetSession(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetSession");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODNULL;
}

SOM_Scope ODBoolean  SOMLINK ODStorageUnitExists(ODStorageUnit *somSelf, Environment *ev,
		ODPropertyName propertyName,
		ODValueType valueType,
		ODValueIndex valueIndex)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitExists");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODNULL;
}

SOM_Scope ODBoolean  SOMLINK ODStorageUnitExistsWithCursor(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitCursor* cursor)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitExistsWithCursor");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODFalse;
}

SOM_Scope ODULong  SOMLINK ODStorageUnitCountProperties(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitCountProperties");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope ODULong  SOMLINK ODStorageUnitCountValues(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitCountValues");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope ODStorageUnit*  SOMLINK ODStorageUnitFocus(ODStorageUnit *somSelf, Environment *ev,
		ODPropertyName propertyName,
		ODPositionCode propertyPosCode,
		ODValueType valueType,
		ODValueIndex valueIndex,
		ODPositionCode valuePosCode)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitFocus");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return somSelf;
}

SOM_Scope ODStorageUnit*  SOMLINK ODStorageUnitFocusWithCursor(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitCursor* cursor)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitFocusWithCursor");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return somSelf;
}

SOM_Scope ODStorageUnit*  SOMLINK ODStorageUnitExternalize(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitExternalize");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return somSelf;
}

SOM_Scope ODStorageUnit*  SOMLINK ODStorageUnitInternalize(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitInternalize");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return somSelf;
}

SOM_Scope ODID  SOMLINK ODStorageUnitGetID(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetID");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope ODStorageUnitName  SOMLINK ODStorageUnitGetName(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetName");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODNULL;
}

SOM_Scope void  SOMLINK ODStorageUnitSetName(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitName name)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitSetName");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope ODStorageUnit*  SOMLINK ODStorageUnitAddProperty(ODStorageUnit *somSelf, Environment *ev,
		ODPropertyName propertyName)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitAddProperty");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return somSelf;
}

SOM_Scope ODStorageUnit*  SOMLINK ODStorageUnitAddValue(ODStorageUnit *somSelf, Environment *ev,
		ODValueType type)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitAddValue");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return somSelf;
}

SOM_Scope ODStorageUnit*  SOMLINK ODStorageUnitRemove(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitRemove");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return somSelf;
}

SOM_Scope void  SOMLINK ODStorageUnitCloneInto(ODStorageUnit *somSelf, Environment *ev,
		ODDraftKey key,
		ODStorageUnit* destStorageUnit,
		ODID initiatingFrame)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitCloneInto");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope ODStorageUnitView*  SOMLINK ODStorageUnitCreateView(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitCreateView");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODNULL;
}

SOM_Scope ODStorageUnitCursor*  SOMLINK ODStorageUnitCreateCursor(ODStorageUnit *somSelf, Environment *ev,
		ODPropertyName propertyName,
		ODValueType valueType,
		ODValueIndex valueIndex)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitCreateCursor");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODNULL;
}

SOM_Scope ODStorageUnitCursor*  SOMLINK ODStorageUnitCreateCursorWithFocus(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitCreateCursorWithFocus");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODNULL;
}

SOM_Scope ODPropertyName  SOMLINK ODStorageUnitGetProperty(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetProperty");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODNULL;
}

SOM_Scope ODValueType  SOMLINK ODStorageUnitGetType(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetType");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODNULL;
}

SOM_Scope void  SOMLINK ODStorageUnitSetType(ODStorageUnit *somSelf, Environment *ev,
		ODValueType valueType)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitSetType");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope void  SOMLINK ODStorageUnitSetOffset(ODStorageUnit *somSelf, Environment *ev,
		ODULong offset)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitSetOffset");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope ODULong  SOMLINK ODStorageUnitGetOffset(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetOffset");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope ODULong  SOMLINK ODStorageUnitGetValue(ODStorageUnit *somSelf, Environment *ev,
	ODULong length,
	ODByteArray* value)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetValue");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope void  SOMLINK ODStorageUnitSetValue(ODStorageUnit *somSelf, Environment *ev,
	ODByteArray* value)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitSetValue");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope void  SOMLINK ODStorageUnitInsertValue(ODStorageUnit *somSelf, Environment *ev,
		ODByteArray* value)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitInsertValue");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope void  SOMLINK ODStorageUnitDeleteValue(ODStorageUnit *somSelf, Environment *ev,
		ODULong length)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitDeleteValue");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope ODULong  SOMLINK ODStorageUnitGetSize(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetSize");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope ODBoolean  SOMLINK ODStorageUnitIsValidStorageUnitRef(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitRef aRef)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitIsValidStorageUnitRef");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODFalse;
}

SOM_Scope   void SOMLINK ODStorageUnitGetStrongStorageUnitRef(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitID embeddedSUID,
		ODStorageUnitRef ref)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetStrongStorageUnitRef");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope void  SOMLINK ODStorageUnitGetWeakStorageUnitRef(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitID embeddedSUID,
		ODStorageUnitRef ref)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetWeakStorageUnitRef");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope ODBoolean  SOMLINK ODStorageUnitIsStrongStorageUnitRef(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitRef ref)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitIsStrongStorageUnitRef");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODFalse;
}

SOM_Scope ODBoolean  SOMLINK ODStorageUnitIsWeakStorageUnitRef(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitRef ref)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitIsWeakStorageUnitRef");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODFalse;
}

SOM_Scope ODStorageUnit*  SOMLINK ODStorageUnitRemoveStorageUnitRef(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitRef aRef)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitRemoveStorageUnitRef");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return somSelf;
}

SOM_Scope ODStorageUnitID  SOMLINK ODStorageUnitGetIDFromStorageUnitRef(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitRef aRef)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetIDFromStorageUnitRef");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope ODStorageUnitRefIterator*  SOMLINK ODStorageUnitCreateStorageUnitRefIterator(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitCreateStorageUnitRefIterator");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODNULL;
}

SOM_Scope void  SOMLINK ODStorageUnitSetStorageUnitRef(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitID embeddedSUID, ODStorageUnitRef ref)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitSetStorageUnitRef");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope ODULong  SOMLINK ODStorageUnitGetGenerationNumber(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetGenerationNumber");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope ODULong  SOMLINK ODStorageUnitIncrementGenerationNumber(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitIncrementGenerationNumber");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope ODBoolean  SOMLINK ODStorageUnitIsPromiseValue(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitIsPromiseValue");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return kODFalse;
}

SOM_Scope void  SOMLINK ODStorageUnitSetPromiseValue(ODStorageUnit *somSelf, Environment *ev,
		ODValueType valueType,
		ODULong offset,
		ODByteArray* value,
		ODPart* sourcePart)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitSetPromiseValue");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope ODULong  SOMLINK ODStorageUnitGetPromiseValue(ODStorageUnit *somSelf, Environment *ev,
		ODValueType valueType,
		ODULong offset,
		ODULong length,
		ODByteArray* value,
		ODPart** sourcePart)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitGetPromiseValue");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope void  SOMLINK ODStorageUnitClearAllPromises(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitClearAllPromises");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope void  SOMLINK ODStorageUnitResolveAllPromises(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitResolveAllPromises");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope ODStorageUnitKey  SOMLINK ODStorageUnitLock(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitKey key)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitLock");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
	
	return 0;
}

SOM_Scope void  SOMLINK ODStorageUnitUnlock(ODStorageUnit *somSelf, Environment *ev,
		ODStorageUnitKey key)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitUnlock");

	WARN("A subclass should have overridden this method!");
	ODSetSOMException(ev,kODErrSubClassResponsibility, "SubClass Responsibility");
}

SOM_Scope void  SOMLINK ODStorageUnitsomUninit(ODStorageUnit *somSelf)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitsomUninit");

    ODStorageUnit_parent_ODRefCntObject_somUninit(somSelf);
}

SOM_Scope void  SOMLINK ODStorageUnitAcquire(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitAcquire");

    ODStorageUnit_parent_ODRefCntObject_Acquire(somSelf,ev);
}

SOM_Scope void  SOMLINK ODStorageUnitRelease(ODStorageUnit *somSelf, Environment *ev)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitRelease");

    ODStorageUnit_parent_ODRefCntObject_Release(somSelf,ev);
}

SOM_Scope ODSize  SOMLINK ODStorageUnitPurge(ODStorageUnit *somSelf, Environment *ev,
		ODSize size)
{
    /* ODStorageUnitData *somThis = ODStorageUnitGetData(somSelf); */
    ODStorageUnitMethodDebug("ODStorageUnit","ODStorageUnitPurge");

	ODSize purged = 0; ODVolatile(purged);
	
	SOM_TRY
		purged = (ODStorageUnit_parent_ODRefCntObject_Purge(somSelf,ev,size));
	SOM_CATCH_ALL
    	WARN("Error %ld trying to purge in ODBaseShape__Purge",ErrorCode());
		SetErrorCode(kODNoError);		// dh - Eat the exception; Purge should not 
										// propagate it because clients function
										// fine whether memory was purged or not.
	SOM_ENDTRY

	return purged;
}
